@@ -4,11 +4,13 @@
 
 #include "base/task/sequence_manager/sequence_manager_impl.h"
 
+#include <atomic>
 #include <queue>
 #include <vector>
 
 #include "base/bind.h"
-#include "base/bit_cast.h"
+#include "base/callback.h"
+#include "base/callback_helpers.h"
 #include "base/compiler_specific.h"
 #include "base/debug/crash_logging.h"
 #include "base/debug/stack_trace.h"
@@ -16,21 +18,26 @@
 #include "base/logging.h"
 #include "base/memory/ptr_util.h"
 #include "base/no_destructor.h"
-#include "base/optional.h"
+#include "base/notreached.h"
+#include "base/observer_list.h"
 #include "base/rand_util.h"
 #include "base/ranges/algorithm.h"
-#include "base/task/sequence_manager/real_time_domain.h"
+#include "base/task/sequence_manager/enqueue_order.h"
 #include "base/task/sequence_manager/task_time_observer.h"
 #include "base/task/sequence_manager/thread_controller_impl.h"
 #include "base/task/sequence_manager/thread_controller_with_message_pump_impl.h"
+#include "base/task/sequence_manager/time_domain.h"
+#include "base/task/sequence_manager/wake_up_queue.h"
 #include "base/task/sequence_manager/work_queue.h"
 #include "base/task/sequence_manager/work_queue_sets.h"
+#include "base/task/task_features.h"
 #include "base/threading/thread_id_name_manager.h"
 #include "base/threading/thread_local.h"
 #include "base/time/default_tick_clock.h"
 #include "base/time/tick_clock.h"
 #include "base/trace_event/base_tracing.h"
 #include "build/build_config.h"
+#include "third_party/abseil-cpp/absl/types/optional.h"
 
 namespace base {
 namespace sequence_manager {
@@ -91,8 +98,7 @@ using TimeRecordingPolicy =
 
 namespace {
 
-constexpr TimeDelta kLongTaskTraceEventThreshold =
-    TimeDelta::FromMilliseconds(50);
+constexpr TimeDelta kLongTaskTraceEventThreshold = Milliseconds(50);
 // Proportion of tasks which will record thread time for metrics.
 const double kTaskSamplingRateForRecordingCPUTime = 0.01;
 // Proprortion of SequenceManagers which will record thread time for each task,
@@ -103,14 +109,15 @@ const double kThreadSamplingRateForRecordingCPUTime = 0.0001;
 // early when detected.
 constexpr int kMemoryCorruptionSentinelValue = 0xdeadbeef;
 
-void ReclaimMemoryFromQueue(internal::TaskQueueImpl* queue,
-                            std::map<TimeDomain*, TimeTicks>* time_domain_now) {
-  TimeDomain* time_domain = queue->GetTimeDomain();
-  if (time_domain_now->find(time_domain) == time_domain_now->end())
-    time_domain_now->insert(std::make_pair(time_domain, time_domain->Now()));
-  queue->ReclaimMemory(time_domain_now->at(time_domain));
-  queue->delayed_work_queue()->RemoveAllCanceledTasksFromFront();
-  queue->immediate_work_queue()->RemoveAllCanceledTasksFromFront();
+void ReclaimMemoryFromQueue(internal::TaskQueueImpl* queue, LazyNow* lazy_now) {
+  queue->ReclaimMemory(lazy_now->Now());
+  // If the queue was shut down as a side-effect of reclaiming memory, |queue|
+  // will still be valid but the work queues will have been removed by
+  // TaskQueueImpl::UnregisterTaskQueue.
+  if (queue->delayed_work_queue()) {
+    queue->delayed_work_queue()->RemoveAllCanceledTasksFromFront();
+    queue->immediate_work_queue()->RemoveAllCanceledTasksFromFront();
+  }
 }
 
 SequenceManager::MetricRecordingSettings InitializeMetricRecordingSettings(
@@ -127,7 +134,7 @@ SequenceManager::MetricRecordingSettings InitializeMetricRecordingSettings(
 // Writes |address| in hexadecimal ("0x11223344") form starting from |output|
 // and moving backwards in memory. Returns a pointer to the first digit of the
 // result. Does *not* NUL-terminate the number.
-#if !defined(OS_NACL)
+#if !BUILDFLAG(IS_NACL)
 char* PrependHexAddress(char* output, const void* address) {
   uintptr_t value = reinterpret_cast<uintptr_t>(address);
   static const char kHexChars[] = "0123456789ABCDEF";
@@ -139,7 +146,14 @@ char* PrependHexAddress(char* output, const void* address) {
   *output = '0';
   return output;
 }
-#endif  // !defined(OS_NACL)
+#endif  // !BUILDFLAG(IS_NACL)
+
+// Controls whether canceled tasks are removed from the front of the queue when
+// deciding when the next wake up should happen.
+// Note: An atomic is used here because some tests can initialize two different
+//       sequence managers on different threads (e.g. by using base::Thread).
+std::atomic_bool g_no_wake_ups_for_canceled_tasks{false};
+std::atomic<TimeDelta> g_task_leeway_SMI{WakeUp::kDefaultLeeway};
 
 }  // namespace
 
@@ -184,6 +198,10 @@ SequenceManagerImpl* SequenceManagerImpl::GetCurrent() {
   return GetTLSSequenceManagerImpl()->Get();
 }
 
+// static
+const Feature SequenceManagerImpl::kNoWakeUpsForCanceledTasks{
+    "NoWakeUpsForCanceledTasks", FEATURE_DISABLED_BY_DEFAULT};
+
 SequenceManagerImpl::SequenceManagerImpl(
     std::unique_ptr<internal::ThreadController> controller,
     SequenceManager::Settings settings)
@@ -196,15 +214,14 @@ SequenceManagerImpl::SequenceManagerImpl(
 
       empty_queues_to_reload_(associated_thread_),
       memory_corruption_sentinel_(kMemoryCorruptionSentinelValue),
-      main_thread_only_(associated_thread_, settings_) {
+      main_thread_only_(this, associated_thread_, settings_, settings_.clock),
+      clock_(settings_.clock) {
   TRACE_EVENT_OBJECT_CREATED_WITH_ID(
       TRACE_DISABLED_BY_DEFAULT("sequence_manager"), "SequenceManager", this);
   main_thread_only().selector.SetTaskQueueSelectorObserver(this);
 
   main_thread_only().next_time_to_reclaim_memory =
-      settings_.clock->NowTicks() + kReclaimMemoryInterval;
-
-  RegisterTimeDomain(main_thread_only().real_time_domain.get());
+      main_thread_clock()->NowTicks() + kReclaimMemoryInterval;
 
   controller_->SetSequencedTaskSource(this);
 }
@@ -214,7 +231,7 @@ SequenceManagerImpl::~SequenceManagerImpl() {
   TRACE_EVENT_OBJECT_DELETED_WITH_ID(
       TRACE_DISABLED_BY_DEFAULT("sequence_manager"), "SequenceManager", this);
 
-#if defined(OS_IOS)
+#if BUILDFLAG(IS_IOS)
   if (settings_.message_loop_type == MessagePumpType::UI &&
       associated_thread_->IsBound()) {
     controller_->DetachFromMessagePump();
@@ -258,13 +275,19 @@ SequenceManagerImpl::~SequenceManagerImpl() {
 }
 
 SequenceManagerImpl::MainThreadOnly::MainThreadOnly(
+    SequenceManagerImpl* sequence_manager,
     const scoped_refptr<AssociatedThreadId>& associated_thread,
-    const SequenceManager::Settings& settings)
+    const SequenceManager::Settings& settings,
+    const base::TickClock* clock)
     : selector(associated_thread, settings),
-      real_time_domain(new internal::RealTimeDomain()) {
+      default_clock(clock),
+      time_domain(nullptr),
+      wake_up_queue(std::make_unique<DefaultWakeUpQueue>(associated_thread,
+                                                         sequence_manager)),
+      non_waking_wake_up_queue(
+          std::make_unique<NonWakingWakeUpQueue>(associated_thread)) {
   if (settings.randomised_sampling_enabled) {
-    random_generator = std::mt19937_64(RandUint64());
-    uniform_distribution = std::uniform_real_distribution<double>(0.0, 1.0);
+    random_generator = base::InsecureRandomGenerator();
   }
 }
 
@@ -298,12 +321,41 @@ std::unique_ptr<SequenceManagerImpl> SequenceManagerImpl::CreateUnbound(
                                             std::move(settings)));
 }
 
+// static
+void SequenceManagerImpl::InitializeFeatures() {
+  ApplyNoWakeUpsForCanceledTasks();
+  TaskQueueImpl::InitializeFeatures();
+  ThreadControllerWithMessagePumpImpl::InitializeFeatures();
+  g_task_leeway_SMI.store(kTaskLeewayParam.Get(), std::memory_order_relaxed);
+}
+
+// static
+void SequenceManagerImpl::ApplyNoWakeUpsForCanceledTasks() {
+  // Since kNoWakeUpsForCanceledTasks is not constexpr (forbidden for Features),
+  // it cannot be used to initialize |g_no_wake_ups_for_canceled_tasks| at
+  // compile time. At least DCHECK that its initial value matches the default
+  // value of the feature here.
+  DCHECK_EQ(
+      g_no_wake_ups_for_canceled_tasks.load(std::memory_order_relaxed),
+      kNoWakeUpsForCanceledTasks.default_state == FEATURE_ENABLED_BY_DEFAULT);
+  g_no_wake_ups_for_canceled_tasks.store(
+      FeatureList::IsEnabled(kNoWakeUpsForCanceledTasks),
+      std::memory_order_relaxed);
+}
+
+// static
+void SequenceManagerImpl::ResetNoWakeUpsForCanceledTasksForTesting() {
+  g_no_wake_ups_for_canceled_tasks.store(
+      kNoWakeUpsForCanceledTasks.default_state == FEATURE_ENABLED_BY_DEFAULT,
+      std::memory_order_relaxed);
+}
+
 void SequenceManagerImpl::BindToMessagePump(std::unique_ptr<MessagePump> pump) {
   controller_->BindToCurrentThread(std::move(pump));
   CompleteInitializationOnBoundThread();
 
   // On Android attach to the native loop when there is one.
-#if defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
   if (settings_.message_loop_type == MessagePumpType::UI ||
       settings_.message_loop_type == MessagePumpType::JAVA) {
     controller_->AttachToMessagePump();
@@ -311,7 +363,7 @@ void SequenceManagerImpl::BindToMessagePump(std::unique_ptr<MessagePump> pump) {
 #endif
 
   // On iOS attach to the native loop when there is one.
-#if defined(OS_IOS)
+#if BUILDFLAG(IS_IOS)
   if (settings_.message_loop_type == MessagePumpType::UI) {
     controller_->AttachToMessagePump();
   }
@@ -323,12 +375,6 @@ void SequenceManagerImpl::BindToCurrentThread() {
   CompleteInitializationOnBoundThread();
 }
 
-void SequenceManagerImpl::BindToCurrentThread(
-    std::unique_ptr<MessagePump> pump) {
-  associated_thread_->BindToCurrentThread();
-  BindToMessagePump(std::move(pump));
-}
-
 scoped_refptr<SequencedTaskRunner>
 SequenceManagerImpl::GetTaskRunnerForCurrentTask() {
   DCHECK_CALLED_ON_VALID_THREAD(associated_thread_->thread_checker);
@@ -349,29 +395,30 @@ void SequenceManagerImpl::CompleteInitializationOnBoundThread() {
   }
 }
 
-void SequenceManagerImpl::RegisterTimeDomain(TimeDomain* time_domain) {
-  main_thread_only().time_domains.insert(time_domain);
-  time_domain->OnRegisterWithSequenceManager(this);
+void SequenceManagerImpl::SetTimeDomain(TimeDomain* time_domain) {
+  DCHECK(!main_thread_only().time_domain);
+  DCHECK(time_domain);
+  time_domain->OnAssignedToSequenceManager(this);
+  controller_->SetTickClock(time_domain);
+  main_thread_only().time_domain = time_domain;
+  clock_.store(time_domain, std::memory_order_release);
 }
 
-void SequenceManagerImpl::UnregisterTimeDomain(TimeDomain* time_domain) {
-  main_thread_only().time_domains.erase(time_domain);
-}
-
-TimeDomain* SequenceManagerImpl::GetRealTimeDomain() const {
-  return main_thread_only().real_time_domain.get();
+void SequenceManagerImpl::ResetTimeDomain() {
+  controller_->SetTickClock(main_thread_only().default_clock);
+  clock_.store(main_thread_only().default_clock, std::memory_order_release);
+  main_thread_only().time_domain = nullptr;
 }
 
 std::unique_ptr<internal::TaskQueueImpl>
 SequenceManagerImpl::CreateTaskQueueImpl(const TaskQueue::Spec& spec) {
   DCHECK_CALLED_ON_VALID_THREAD(associated_thread_->thread_checker);
-  TimeDomain* time_domain = spec.time_domain
-                                ? spec.time_domain
-                                : main_thread_only().real_time_domain.get();
-  DCHECK(main_thread_only().time_domains.find(time_domain) !=
-         main_thread_only().time_domains.end());
   std::unique_ptr<internal::TaskQueueImpl> task_queue =
-      std::make_unique<internal::TaskQueueImpl>(this, time_domain, spec);
+      std::make_unique<internal::TaskQueueImpl>(
+          this,
+          spec.non_waking ? main_thread_only().non_waking_wake_up_queue.get()
+                          : main_thread_only().wake_up_queue.get(),
+          spec);
   main_thread_only().active_queues.insert(task_queue.get());
   main_thread_only().selector.AddQueue(task_queue.get());
   return task_queue;
@@ -437,14 +484,12 @@ void SequenceManagerImpl::MoveReadyDelayedTasksToWorkQueues(LazyNow* lazy_now) {
   TRACE_EVENT0(TRACE_DISABLED_BY_DEFAULT("sequence_manager"),
                "SequenceManagerImpl::MoveReadyDelayedTasksToWorkQueues");
 
-  for (TimeDomain* time_domain : main_thread_only().time_domains) {
-    if (time_domain == main_thread_only().real_time_domain.get()) {
-      time_domain->MoveReadyDelayedTasksToWorkQueues(lazy_now);
-    } else {
-      LazyNow time_domain_lazy_now = time_domain->CreateLazyNow();
-      time_domain->MoveReadyDelayedTasksToWorkQueues(&time_domain_lazy_now);
-    }
-  }
+  EnqueueOrder delayed_task_group_enqueue_order = GetNextSequenceNumber();
+  main_thread_only().wake_up_queue->MoveReadyDelayedTasksToWorkQueues(
+      lazy_now, delayed_task_group_enqueue_order);
+  main_thread_only()
+      .non_waking_wake_up_queue->MoveReadyDelayedTasksToWorkQueues(
+          lazy_now, delayed_task_group_enqueue_order);
 }
 
 void SequenceManagerImpl::OnBeginNestedRunLoop() {
@@ -457,10 +502,12 @@ void SequenceManagerImpl::OnExitNestedRunLoop() {
   main_thread_only().nesting_depth--;
   DCHECK_GE(main_thread_only().nesting_depth, 0);
   if (main_thread_only().nesting_depth == 0) {
-    // While we were nested some non-nestable tasks may have been deferred.
-    // We push them back onto the *front* of their original work queues,
-    // that's why we iterate |non_nestable_task_queue| in FIFO order.
-    LazyNow exited_nested_now(controller_->GetClock());
+    // While we were nested some non-nestable tasks may have been deferred. We
+    // push them back onto the *front* of their original work queues, that's why
+    // we iterate |non_nestable_task_queue| in LIFO order (we want
+    // |non_nestable_task.front()| to be the last task pushed at the front of
+    // |task_queue|).
+    LazyNow exited_nested_now(main_thread_clock());
     while (!main_thread_only().non_nestable_task_queue.empty()) {
       internal::TaskQueueImpl::DeferredNonNestableTask& non_nestable_task =
           main_thread_only().non_nestable_task_queue.back();
@@ -485,9 +532,14 @@ void SequenceManagerImpl::ScheduleWork() {
   controller_->ScheduleWork();
 }
 
-void SequenceManagerImpl::SetNextDelayedDoWork(LazyNow* lazy_now,
-                                               TimeTicks run_time) {
-  controller_->SetNextDelayedDoWork(lazy_now, run_time);
+void SequenceManagerImpl::SetNextWakeUp(LazyNow* lazy_now,
+                                        absl::optional<WakeUp> wake_up) {
+  auto next_wake_up = AdjustWakeUp(wake_up, lazy_now);
+  if (next_wake_up && next_wake_up->is_immediate()) {
+    ScheduleWork();
+  } else {
+    controller_->SetNextDelayedDoWork(lazy_now, next_wake_up);
+  }
 }
 
 namespace {
@@ -516,10 +568,11 @@ const char* RunTaskTraceNameForPriority(TaskQueue::QueuePriority priority) {
 
 }  // namespace
 
-Task* SequenceManagerImpl::SelectNextTask(SelectTaskOption option) {
-  Task* task = SelectNextTaskImpl(option);
-  if (!task)
-    return nullptr;
+absl::optional<SequenceManagerImpl::SelectedTask>
+SequenceManagerImpl::SelectNextTask(SelectTaskOption option) {
+  absl::optional<SelectedTask> selected_task = SelectNextTaskImpl(option);
+  if (!selected_task)
+    return selected_task;
 
   ExecutingTask& executing_task =
       *main_thread_only().task_execution_stack.rbegin();
@@ -531,10 +584,10 @@ Task* SequenceManagerImpl::SelectNextTask(SelectTaskOption option) {
                      "task_type", executing_task.task_type);
   TRACE_EVENT_BEGIN0("sequence_manager", executing_task.task_queue_name);
 
-  return task;
+  return selected_task;
 }
 
-#if DCHECK_IS_ON() && !defined(OS_NACL)
+#if DCHECK_IS_ON() && !BUILDFLAG(IS_NACL)
 void SequenceManagerImpl::LogTaskDebugInfo(
     const WorkQueue* selected_work_queue) const {
   const Task* task = selected_work_queue->GetFrontTask();
@@ -586,9 +639,10 @@ void SequenceManagerImpl::LogTaskDebugInfo(
     }
   }
 }
-#endif  // DCHECK_IS_ON() && !defined(OS_NACL)
+#endif  // DCHECK_IS_ON() && !BUILDFLAG(IS_NACL)
 
-Task* SequenceManagerImpl::SelectNextTaskImpl(SelectTaskOption option) {
+absl::optional<SequenceManagerImpl::SelectedTask>
+SequenceManagerImpl::SelectNextTaskImpl(SelectTaskOption option) {
   CHECK(Validate());
 
   DCHECK_CALLED_ON_VALID_THREAD(associated_thread_->thread_checker);
@@ -596,7 +650,7 @@ Task* SequenceManagerImpl::SelectNextTaskImpl(SelectTaskOption option) {
                "SequenceManagerImpl::SelectNextTask");
 
   ReloadEmptyWorkQueues();
-  LazyNow lazy_now(controller_->GetClock());
+  LazyNow lazy_now(main_thread_clock());
   MoveReadyDelayedTasksToWorkQueues(&lazy_now);
 
   // If we sampled now, check if it's time to reclaim memory next time we go
@@ -616,7 +670,7 @@ Task* SequenceManagerImpl::SelectNextTaskImpl(SelectTaskOption option) {
                                             /* force_verbose */ false));
 
     if (!work_queue)
-      return nullptr;
+      return absl::nullopt;
 
     // If the head task was canceled, remove it and run the selector again.
     if (UNLIKELY(work_queue->RemoveAllCanceledTasksFromFront()))
@@ -641,12 +695,12 @@ Task* SequenceManagerImpl::SelectNextTaskImpl(SelectTaskOption option) {
             work_queue->task_queue()->GetQueuePriority()))) {
       TRACE_EVENT0(TRACE_DISABLED_BY_DEFAULT("sequence_manager"),
                    "SequenceManager.YieldToNative");
-      return nullptr;
+      return absl::nullopt;
     }
 
-#if DCHECK_IS_ON() && !defined(OS_NACL)
+#if DCHECK_IS_ON() && !BUILDFLAG(IS_NACL)
     LogTaskDebugInfo(work_queue);
-#endif  // DCHECK_IS_ON() && !defined(OS_NACL)
+#endif  // DCHECK_IS_ON() && !BUILDFLAG(IS_NACL)
 
     main_thread_only().task_execution_stack.emplace_back(
         work_queue->TakeTaskFromWorkQueue(), work_queue->task_queue(),
@@ -656,7 +710,9 @@ Task* SequenceManagerImpl::SelectNextTaskImpl(SelectTaskOption option) {
         *main_thread_only().task_execution_stack.rbegin();
     NotifyWillProcessTask(&executing_task, &lazy_now);
 
-    return &executing_task.pending_task;
+    return SelectedTask(
+        executing_task.pending_task,
+        executing_task.task_queue->task_execution_trace_logger());
   }
 }
 
@@ -666,7 +722,7 @@ bool SequenceManagerImpl::ShouldRunTaskOfPriority(
 }
 
 void SequenceManagerImpl::DidRunTask() {
-  LazyNow lazy_now(controller_->GetClock());
+  LazyNow lazy_now(main_thread_clock());
   ExecutingTask& executing_task =
       *main_thread_only().task_execution_stack.rbegin();
 
@@ -681,7 +737,19 @@ void SequenceManagerImpl::DidRunTask() {
     CleanUpQueues();
 }
 
-TimeDelta SequenceManagerImpl::DelayTillNextTask(
+void SequenceManagerImpl::RemoveAllCanceledDelayedTasksFromFront(
+    LazyNow* lazy_now) {
+  if (!g_no_wake_ups_for_canceled_tasks.load(std::memory_order_relaxed))
+    return;
+
+  main_thread_only().wake_up_queue->RemoveAllCanceledDelayedTasksFromFront(
+      lazy_now);
+  main_thread_only()
+      .non_waking_wake_up_queue->RemoveAllCanceledDelayedTasksFromFront(
+          lazy_now);
+}
+
+absl::optional<WakeUp> SequenceManagerImpl::GetPendingWakeUp(
     LazyNow* lazy_now,
     SelectTaskOption option) const {
   DCHECK_CALLED_ON_VALID_THREAD(associated_thread_->thread_checker);
@@ -692,8 +760,8 @@ TimeDelta SequenceManagerImpl::DelayTillNextTask(
     // work to be done. However we may want to yield to native work if it is
     // more important.
     if (UNLIKELY(!ShouldRunTaskOfPriority(*priority)))
-      return GetDelayTillNextDelayedTask(lazy_now, option);
-    return TimeDelta();
+      return AdjustWakeUp(GetNextDelayedWakeUpWithOption(option), lazy_now);
+    return WakeUp{};
   }
 
   // There may be some incoming immediate work which we haven't accounted for.
@@ -704,55 +772,72 @@ TimeDelta SequenceManagerImpl::DelayTillNextTask(
   if (auto priority =
           main_thread_only().selector.GetHighestPendingPriority(option)) {
     if (UNLIKELY(!ShouldRunTaskOfPriority(*priority)))
-      return GetDelayTillNextDelayedTask(lazy_now, option);
-    return TimeDelta();
+      return AdjustWakeUp(GetNextDelayedWakeUpWithOption(option), lazy_now);
+    return WakeUp{};
   }
 
   // Otherwise we need to find the shortest delay, if any.  NB we don't need to
   // call MoveReadyDelayedTasksToWorkQueues because it's assumed
   // DelayTillNextTask will return TimeDelta>() if the delayed task is due to
   // run now.
-  return GetDelayTillNextDelayedTask(lazy_now, option);
+  return AdjustWakeUp(GetNextDelayedWakeUpWithOption(option), lazy_now);
 }
 
-TimeDelta SequenceManagerImpl::GetDelayTillNextDelayedTask(
-    LazyNow* lazy_now,
+absl::optional<WakeUp> SequenceManagerImpl::GetNextDelayedWakeUp() const {
+  DCHECK_CALLED_ON_VALID_THREAD(associated_thread_->thread_checker);
+  return main_thread_only().wake_up_queue->GetNextDelayedWakeUp();
+}
+
+absl::optional<WakeUp> SequenceManagerImpl::GetNextDelayedWakeUpWithOption(
     SelectTaskOption option) const {
   DCHECK_CALLED_ON_VALID_THREAD(associated_thread_->thread_checker);
 
   if (option == SelectTaskOption::kSkipDelayedTask)
-    return TimeDelta::Max();
+    return absl::nullopt;
+  return GetNextDelayedWakeUp();
+}
 
-  TimeDelta delay_till_next_task = TimeDelta::Max();
-  for (TimeDomain* time_domain : main_thread_only().time_domains) {
-    Optional<TimeDelta> delay = time_domain->DelayTillNextTask(lazy_now);
-    if (!delay)
-      continue;
+absl::optional<WakeUp> SequenceManagerImpl::AdjustWakeUp(
+    absl::optional<WakeUp> wake_up,
+    LazyNow* lazy_now) const {
+  DCHECK_CALLED_ON_VALID_THREAD(associated_thread_->thread_checker);
+  if (!wake_up)
+    return absl::nullopt;
+  // Overdue work needs to be run immediately.
+  if (lazy_now->Now() >= wake_up->earliest_time())
+    return WakeUp{};
+  // If |time_domain| is present, we don't want an actual OS level delayed wake
+  // up scheduled, so pretend we have no more work. This will result in
+  // appearing idle and |time_domain| will decide what to do in
+  // MaybeFastForwardToWakeUp().
+  if (main_thread_only().time_domain)
+    return absl::nullopt;
+  return *wake_up;
+}
 
-    if (*delay < delay_till_next_task)
-      delay_till_next_task = *delay;
-  }
-  return delay_till_next_task;
+void SequenceManagerImpl::MaybeAddLeewayToTask(Task& task) const {
+  if (!main_thread_only().time_domain)
+    task.leeway = g_task_leeway_SMI.load(std::memory_order_relaxed);
 }
 
 bool SequenceManagerImpl::HasPendingHighResolutionTasks() {
-  for (TimeDomain* time_domain : main_thread_only().time_domains) {
-    if (time_domain->has_pending_high_resolution_tasks())
-      return true;
-  }
-  return false;
+  // Only consider high-res tasks in the |wake_up_queue| (ignore the
+  // |non_waking_wake_up_queue|).
+  return main_thread_only().wake_up_queue->has_pending_high_resolution_tasks();
 }
 
 bool SequenceManagerImpl::OnSystemIdle() {
+  auto wakeup = main_thread_only().wake_up_queue->GetNextDelayedWakeUp();
   bool have_work_to_do = false;
-  for (TimeDomain* time_domain : main_thread_only().time_domains) {
-    if (time_domain->MaybeFastForwardToNextTask(
-            controller_->ShouldQuitRunLoopWhenIdle())) {
-      have_work_to_do = true;
-    }
+  if (main_thread_only().time_domain) {
+    have_work_to_do = main_thread_only().time_domain->MaybeFastForwardToWakeUp(
+        wakeup, controller_->ShouldQuitRunLoopWhenIdle());
   }
-  if (!have_work_to_do)
+  if (!have_work_to_do) {
     MaybeReclaimMemory();
+    if (main_thread_only().on_next_idle_callback)
+      std::move(main_thread_only().on_next_idle_callback).Run();
+  }
   return have_work_to_do;
 }
 
@@ -795,6 +880,10 @@ void SequenceManagerImpl::NotifyWillProcessTask(ExecutingTask* executing_task,
   if (recording_policy == TimeRecordingPolicy::DoRecord)
     executing_task->task_timing.RecordTaskStart(time_before_task);
 
+  // Maybe invalidate the delayed task handle. |pending_task| is guaranteed to
+  // be valid here (not canceled).
+  executing_task->pending_task.WillRunTask();
+
   if (!executing_task->task_queue->GetShouldNotifyObservers())
     return;
 
@@ -975,10 +1064,12 @@ Value SequenceManagerImpl::AsValueWithSelectorResult(
   state.SetStringKey("native_work_priority",
                      TaskQueue::PriorityToString(
                          *main_thread_only().pending_native_work.begin()));
-  Value time_domains(Value::Type::LIST);
-  for (auto* time_domain : main_thread_only().time_domains)
-    time_domains.Append(time_domain->AsValue());
-  state.SetKey("time_domains", std::move(time_domains));
+  state.SetKey("time_domain", main_thread_only().time_domain
+                                  ? main_thread_only().time_domain->AsValue()
+                                  : Value());
+  state.SetKey("wake_up_queue", main_thread_only().wake_up_queue->AsValue(now));
+  state.SetKey("non_waking_wake_up_queue",
+               main_thread_only().non_waking_wake_up_queue->AsValue(now));
   return state;
 }
 
@@ -986,7 +1077,8 @@ void SequenceManagerImpl::OnTaskQueueEnabled(internal::TaskQueueImpl* queue) {
   DCHECK_CALLED_ON_VALID_THREAD(associated_thread_->thread_checker);
   DCHECK(queue->IsQueueEnabled());
   // Only schedule DoWork if there's something to do.
-  if (queue->HasTaskToRunImmediately() && !queue->BlockedByFence())
+  if (queue->HasTaskToRunImmediatelyOrReadyDelayedTask() &&
+      !queue->BlockedByFence())
     ScheduleWork();
 }
 
@@ -1004,11 +1096,18 @@ void SequenceManagerImpl::MaybeReclaimMemory() {
 }
 
 void SequenceManagerImpl::ReclaimMemory() {
-  std::map<TimeDomain*, TimeTicks> time_domain_now;
-  for (auto* const queue : main_thread_only().active_queues)
-    ReclaimMemoryFromQueue(queue, &time_domain_now);
-  for (const auto& pair : main_thread_only().queues_to_gracefully_shutdown)
-    ReclaimMemoryFromQueue(pair.first, &time_domain_now);
+  LazyNow lazy_now(main_thread_clock());
+  for (auto it = main_thread_only().active_queues.begin();
+       it != main_thread_only().active_queues.end();) {
+    auto* const queue = *it++;
+    ReclaimMemoryFromQueue(queue, &lazy_now);
+  }
+  for (auto it = main_thread_only().queues_to_gracefully_shutdown.begin();
+       it != main_thread_only().queues_to_gracefully_shutdown.end();) {
+    auto* const queue = it->first;
+    it++;
+    ReclaimMemoryFromQueue(queue, &lazy_now);
+  }
 }
 
 void SequenceManagerImpl::CleanUpQueues() {
@@ -1042,19 +1141,18 @@ void SequenceManagerImpl::SetDefaultTaskRunner(
 }
 
 const TickClock* SequenceManagerImpl::GetTickClock() const {
-  return controller_->GetClock();
+  return any_thread_clock();
 }
 
 TimeTicks SequenceManagerImpl::NowTicks() const {
-  return controller_->GetClock()->NowTicks();
+  return any_thread_clock()->NowTicks();
 }
 
 bool SequenceManagerImpl::ShouldRecordCPUTimeForTask() {
   DCHECK(ThreadTicks::IsSupported() ||
          !metric_recording_settings_.records_cpu_time_for_some_tasks());
   return metric_recording_settings_.records_cpu_time_for_some_tasks() &&
-         main_thread_only().uniform_distribution(
-             main_thread_only().random_generator) <
+         main_thread_only().random_generator->RandDouble() <
              metric_recording_settings_
                  .task_sampling_rate_for_recording_cpu_time;
 }
@@ -1064,42 +1162,6 @@ SequenceManagerImpl::GetMetricRecordingSettings() const {
   return metric_recording_settings_;
 }
 
-// TODO(altimin): Ensure that this removes all pending tasks.
-void SequenceManagerImpl::DeletePendingTasks() {
-  DCHECK(main_thread_only().task_execution_stack.empty())
-      << "Tasks should be deleted outside RunLoop";
-
-  for (TaskQueueImpl* task_queue : main_thread_only().active_queues)
-    task_queue->DeletePendingTasks();
-  for (const auto& it : main_thread_only().queues_to_gracefully_shutdown)
-    it.first->DeletePendingTasks();
-  for (const auto& it : main_thread_only().queues_to_delete)
-    it.first->DeletePendingTasks();
-}
-
-bool SequenceManagerImpl::HasTasks() {
-  DCHECK_CALLED_ON_VALID_THREAD(associated_thread_->thread_checker);
-  RemoveAllCanceledTasksFromFrontOfWorkQueues();
-
-  for (TaskQueueImpl* task_queue : main_thread_only().active_queues) {
-    if (task_queue->HasTasks())
-      return true;
-  }
-  for (const auto& it : main_thread_only().queues_to_gracefully_shutdown) {
-    if (it.first->HasTasks())
-      return true;
-  }
-  for (const auto& it : main_thread_only().queues_to_delete) {
-    if (it.first->HasTasks())
-      return true;
-  }
-  return false;
-}
-
-MessagePumpType SequenceManagerImpl::GetType() const {
-  return settings_.message_loop_type;
-}
-
 void SequenceManagerImpl::SetTaskExecutionAllowed(bool allowed) {
   controller_->SetTaskExecutionAllowed(allowed);
 }
@@ -1108,7 +1170,7 @@ bool SequenceManagerImpl::IsTaskExecutionAllowed() const {
   return controller_->IsTaskExecutionAllowed();
 }
 
-#if defined(OS_IOS)
+#if BUILDFLAG(IS_IOS)
 void SequenceManagerImpl::AttachToMessagePump() {
   return controller_->AttachToMessagePump();
 }
@@ -1145,6 +1207,11 @@ std::unique_ptr<NativeWorkHandle> SequenceManagerImpl::OnNativeWorkPending(
   return std::make_unique<NativeWorkHandleImpl>(this, priority);
 }
 
+void SequenceManagerImpl::PrioritizeYieldingToNative(
+    base::TimeTicks prioritize_until) {
+  controller_->PrioritizeYieldingToNative(prioritize_until);
+}
+
 void SequenceManagerImpl::AddDestructionObserver(
     CurrentThread::DestructionObserver* destruction_observer) {
   main_thread_only().destruction_observers.AddObserver(destruction_observer);
@@ -1155,6 +1222,12 @@ void SequenceManagerImpl::RemoveDestructionObserver(
   main_thread_only().destruction_observers.RemoveObserver(destruction_observer);
 }
 
+void SequenceManagerImpl::RegisterOnNextIdleCallback(
+    OnceClosure on_next_idle_callback) {
+  DCHECK(!main_thread_only().on_next_idle_callback);
+  main_thread_only().on_next_idle_callback = std::move(on_next_idle_callback);
+}
+
 void SequenceManagerImpl::SetTaskRunner(
     scoped_refptr<SingleThreadTaskRunner> task_runner) {
   controller_->SetDefaultTaskRunner(task_runner);
@@ -1182,17 +1255,17 @@ NOINLINE bool SequenceManagerImpl::Validate() {
 
 void SequenceManagerImpl::EnableCrashKeys(const char* async_stack_crash_key) {
   DCHECK(!main_thread_only().async_stack_crash_key);
-#if !defined(OS_NACL)
+#if !BUILDFLAG(IS_NACL)
   main_thread_only().async_stack_crash_key = debug::AllocateCrashKeyString(
       async_stack_crash_key, debug::CrashKeySize::Size64);
   static_assert(sizeof(main_thread_only().async_stack_buffer) ==
                     static_cast<size_t>(debug::CrashKeySize::Size64),
                 "Async stack buffer size must match crash key size.");
-#endif  // OS_NACL
+#endif  // BUILDFLAG(IS_NACL)
 }
 
 void SequenceManagerImpl::RecordCrashKeys(const PendingTask& pending_task) {
-#if !defined(OS_NACL)
+#if !BUILDFLAG(IS_NACL)
   // SetCrashKeyString is a no-op even if the crash key is null, but we'd still
   // have construct the StringPiece that is passed in.
   if (!main_thread_only().async_stack_crash_key)
@@ -1206,7 +1279,7 @@ void SequenceManagerImpl::RecordCrashKeys(const PendingTask& pending_task) {
   // this.
   //
   // See
-  // https://chromium.googlesource.com/chromium/src/+/master/docs/debugging_with_crash_keys.md
+  // https://chromium.googlesource.com/chromium/src/+/main/docs/debugging_with_crash_keys.md
   // for instructions for symbolizing these crash keys.
   //
   // TODO(skyostil): Find a way to extract the destination function address
@@ -1222,7 +1295,7 @@ void SequenceManagerImpl::RecordCrashKeys(const PendingTask& pending_task) {
   DCHECK_GE(pos, buffer);
   debug::SetCrashKeyString(main_thread_only().async_stack_crash_key,
                            StringPiece(pos, buffer_end - pos));
-#endif  // OS_NACL
+#endif  // BUILDFLAG(IS_NACL)
 }
 
 internal::TaskQueueImpl* SequenceManagerImpl::currently_executing_task_queue()
