@@ -8,24 +8,34 @@
 #include <utility>
 
 #include "base/command_line.h"
+#include "base/memory/raw_ptr.h"
 #include "base/memory/ref_counted.h"
 #include "base/path_service.h"
 #include "base/strings/stringprintf.h"
+#include "base/test/metrics/histogram_tester.h"
 #include "base/test/scoped_mock_time_message_loop_task_runner.h"
 #include "base/test/simple_test_tick_clock.h"
+#include "base/test/values_test_util.h"
+#include "build/chromeos_buildflags.h"
+#include "chrome/browser/browser_process.h"
 #include "chrome/browser/extensions/api/debugger/debugger_api.h"
 #include "chrome/browser/extensions/api/debugger/extension_dev_tools_infobar_delegate.h"
 #include "chrome/browser/extensions/extension_apitest.h"
 #include "chrome/browser/extensions/extension_function_test_utils.h"
 #include "chrome/browser/extensions/extension_management_test_util.h"
+#include "chrome/browser/profiles/profile_destroyer.h"
+#include "chrome/browser/profiles/profile_manager.h"
 #include "chrome/browser/ui/tabs/tab_strip_model.h"
 #include "chrome/common/chrome_paths.h"
 #include "chrome/common/chrome_switches.h"
+#include "chrome/common/pref_names.h"
+#include "chrome/test/base/testing_profile.h"
 #include "chrome/test/base/ui_test_utils.h"
 #include "components/infobars/content/content_infobar_manager.h"
 #include "components/infobars/core/infobar.h"
 #include "components/infobars/core/infobar_delegate.h"
 #include "components/policy/core/common/mock_configuration_policy_provider.h"
+#include "components/prefs/pref_service.h"
 #include "components/sessions/content/session_tab_helper.h"
 #include "content/public/test/browser_test.h"
 #include "content/public/test/browser_test_utils.h"
@@ -39,8 +49,14 @@
 #include "extensions/test/test_extension_dir.h"
 #include "net/dns/mock_host_resolver.h"
 
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+#include "chrome/browser/ash/profiles/profile_helper.h"
+#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
+
 namespace extensions {
 
+using testing::Eq;
+
 class DebuggerApiTest : public ExtensionApiTest {
  protected:
   ~DebuggerApiTest() override = default;
@@ -65,7 +81,7 @@ class DebuggerApiTest : public ExtensionApiTest {
 
   // The command-line for the test process, preserved in order to modify
   // mid-test.
-  base::CommandLine* command_line_;
+  raw_ptr<base::CommandLine> command_line_;
 
   // A basic extension with the debugger permission.
   scoped_refptr<const Extension> extension_;
@@ -99,7 +115,7 @@ void DebuggerApiTest::SetUpOnMainThread() {
 
 testing::AssertionResult DebuggerApiTest::RunAttachFunction(
     const GURL& url, const std::string& expected_error) {
-  ui_test_utils::NavigateToURL(browser(), url);
+  EXPECT_TRUE(ui_test_utils::NavigateToURL(browser(), url));
   content::WebContents* web_contents =
       browser()->tab_strip_model()->GetActiveWebContents();
 
@@ -117,16 +133,17 @@ testing::AssertionResult DebuggerApiTest::RunAttachFunction(
   std::unique_ptr<base::Value> value(
       extension_function_test_utils::RunFunctionAndReturnSingleResult(
           get_targets_function.get(), "[]", browser()));
-  base::ListValue* targets = nullptr;
-  EXPECT_TRUE(value->GetAsList(&targets));
+  EXPECT_TRUE(value->is_list());
+  const base::ListValue& targets = base::Value::AsListValue(*value);
 
   std::string debugger_target_id;
-  for (size_t i = 0; i < targets->GetSize(); ++i) {
-    base::DictionaryValue* target_dict = nullptr;
-    EXPECT_TRUE(targets->GetDictionary(i, &target_dict));
-    int id = -1;
-    if (target_dict->GetInteger("tabId", &id) && id == tab_id) {
-      EXPECT_TRUE(target_dict->GetString("id", &debugger_target_id));
+  for (const base::Value& target_value : targets.GetListDeprecated()) {
+    EXPECT_TRUE(target_value.is_dict());
+    absl::optional<int> id = target_value.FindIntKey("tabId");
+    if (id == tab_id) {
+      const base::DictionaryValue& target_dict =
+          base::Value::AsDictionaryValue(target_value);
+      EXPECT_TRUE(target_dict.GetString("id", &debugger_target_id));
       break;
     }
   }
@@ -380,6 +397,153 @@ IN_PROC_BROWSER_TEST_F(DebuggerApiTest, InfoBarIsRemovedAfterFiveSeconds) {
   EXPECT_EQ(0u, manager->infobar_count());
 }
 
+class CrossProfileDebuggerApiTest : public DebuggerApiTest {
+ protected:
+  Profile* other_profile() { return other_profile_; }
+  Profile* otr_profile() { return otr_profile_; }
+
+  std::unique_ptr<content::WebContents> CreateTabWithProfileAndNavigate(
+      Profile* profile,
+      const GURL& url) {
+    auto wc = content::WebContents::Create(
+        content::WebContents::CreateParams(profile));
+    EXPECT_TRUE(content::NavigateToURL(wc.get(), url));
+    return wc;
+  }
+
+ private:
+  void SetUpOnMainThread() override {
+    ASSERT_TRUE(embedded_test_server()->Start());
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+    ash::ProfileHelper::SetAlwaysReturnPrimaryUserForTesting(true);
+#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
+    DebuggerApiTest::SetUpOnMainThread();
+    profile_manager_ = g_browser_process->profile_manager();
+
+    base::RunLoop run_loop;
+    profile_manager_->CreateProfileAsync(
+        profile_manager_->GenerateNextProfileDirectoryPath(),
+        base::BindRepeating(
+            [](CrossProfileDebuggerApiTest* self, base::RunLoop* run_loop,
+               Profile* profile, Profile::CreateStatus status) {
+              if (status == Profile::CREATE_STATUS_INITIALIZED) {
+                self->other_profile_ = profile;
+                run_loop->Quit();
+              }
+            },
+            this, &run_loop));
+    run_loop.Run();
+    otr_profile_ = profile()->GetPrimaryOTRProfile(true);
+  }
+
+  void TearDownOnMainThread() override {
+    ProfileDestroyer::DestroyProfileWhenAppropriate(otr_profile_);
+    DebuggerApiTest::TearDownOnMainThread();
+  }
+
+  ProfileManager* profile_manager_ = nullptr;
+  Profile* other_profile_ = nullptr;
+  Profile* otr_profile_ = nullptr;
+};
+
+IN_PROC_BROWSER_TEST_F(CrossProfileDebuggerApiTest, GetTargets) {
+  auto wc1 = CreateTabWithProfileAndNavigate(
+      other_profile(),
+      embedded_test_server()->GetURL("/simple.html?other_profile"));
+  auto wc2 = CreateTabWithProfileAndNavigate(
+      otr_profile(),
+      embedded_test_server()->GetURL("/simple.html?off_the_record"));
+
+  {
+    auto get_targets_function =
+        base::MakeRefCounted<DebuggerGetTargetsFunction>();
+    base::Value value = std::move(
+        *extension_function_test_utils::RunFunctionAndReturnSingleResult(
+            get_targets_function.get(), "[]", browser()));
+
+    ASSERT_TRUE(value.is_list());
+    const base::Value::List targets = std::move(value.GetList());
+    ASSERT_THAT(targets, testing::SizeIs(1));
+    EXPECT_THAT(targets[0], base::test::DictionaryHasValue(
+                                "url", base::Value("about:blank")));
+  }
+
+  {
+    auto get_targets_function =
+        base::MakeRefCounted<DebuggerGetTargetsFunction>();
+    base::Value value = std::move(
+        *extension_function_test_utils::RunFunctionAndReturnSingleResult(
+            get_targets_function.get(), "[]", browser(),
+            api_test_utils::RunFunctionFlags::INCLUDE_INCOGNITO));
+
+    ASSERT_TRUE(value.is_list());
+    const base::Value::List targets = std::move(value.GetList());
+    std::vector<std::string> urls;
+    std::transform(
+        targets.begin(), targets.end(), std::back_inserter(urls),
+        [](const base::Value& value) -> std::string {
+          GURL::Replacements remove_port;
+          remove_port.ClearPort();
+          const std::string* url = value.GetDict().FindString("url");
+          return url ? GURL(*url).ReplaceComponents(remove_port).spec()
+                     : "<missing field>";
+        });
+    EXPECT_THAT(urls, testing::UnorderedElementsAre(
+                          "about:blank",
+                          "http://127.0.0.1/simple.html?off_the_record"));
+  }
+}
+
+IN_PROC_BROWSER_TEST_F(CrossProfileDebuggerApiTest, Attach) {
+  auto wc1 = CreateTabWithProfileAndNavigate(
+      other_profile(),
+      embedded_test_server()->GetURL("/simple.html?other_profile"));
+  std::string target_in_other_profile = base::StringPrintf(
+      "[{\"targetId\": \"%s\"}, \"1.1\"]",
+      content::DevToolsAgentHost::GetOrCreateFor(wc1.get())->GetId().c_str());
+
+  {
+    auto debugger_attach_function =
+        base::MakeRefCounted<DebuggerAttachFunction>();
+    debugger_attach_function->set_extension(extension());
+    EXPECT_FALSE(extension_function_test_utils::RunFunction(
+        debugger_attach_function.get(), target_in_other_profile, browser(),
+        api_test_utils::NONE));
+  }
+  {
+    auto debugger_attach_function =
+        base::MakeRefCounted<DebuggerAttachFunction>();
+    debugger_attach_function->set_extension(extension());
+    EXPECT_FALSE(extension_function_test_utils::RunFunction(
+        debugger_attach_function.get(), target_in_other_profile.c_str(),
+        browser(), api_test_utils::INCLUDE_INCOGNITO));
+  }
+
+  auto wc2 = CreateTabWithProfileAndNavigate(
+      otr_profile(),
+      embedded_test_server()->GetURL("/simple.html?off_the_record"));
+  std::string target_in_otr_profile = base::StringPrintf(
+      "[{\"targetId\": \"%s\"}, \"1.1\"]",
+      content::DevToolsAgentHost::GetOrCreateFor(wc2.get())->GetId().c_str());
+
+  {
+    auto debugger_attach_function =
+        base::MakeRefCounted<DebuggerAttachFunction>();
+    debugger_attach_function->set_extension(extension());
+    EXPECT_FALSE(extension_function_test_utils::RunFunction(
+        debugger_attach_function.get(), target_in_otr_profile.c_str(),
+        browser(), api_test_utils::NONE));
+  }
+  {
+    auto debugger_attach_function =
+        base::MakeRefCounted<DebuggerAttachFunction>();
+    debugger_attach_function->set_extension(extension());
+    EXPECT_TRUE(extension_function_test_utils::RunFunction(
+        debugger_attach_function.get(), target_in_otr_profile.c_str(),
+        browser(), api_test_utils::INCLUDE_INCOGNITO));
+  }
+}
+
 IN_PROC_BROWSER_TEST_F(DebuggerApiTest,
                        InfoBarIsNotRemovedIfAttachAgainBeforeFiveSeconds) {
   int tab_id = sessions::SessionTabHelper::IdForTab(
@@ -476,6 +640,10 @@ IN_PROC_BROWSER_TEST_F(DebuggerExtensionApiTest, AttachToEmptyUrls) {
   ASSERT_TRUE(RunExtensionTest("debugger_attach_to_empty_urls")) << message_;
 }
 
+IN_PROC_BROWSER_TEST_F(DebuggerExtensionApiTest, AttachToPdf) {
+  ASSERT_TRUE(RunExtensionTest("debugger_attach_to_pdf")) << message_;
+}
+
 // Tests that navigation to a forbidden URL is properly denied and
 // does not cause a crash.
 // This is a regression test for https://crbug.com/1188889.
@@ -485,6 +653,27 @@ IN_PROC_BROWSER_TEST_F(DebuggerExtensionApiTest, NavigateToForbiddenUrl) {
       << message_;
 }
 
+IN_PROC_BROWSER_TEST_F(DebuggerExtensionApiTest, IsDeveloperModeTrueHistogram) {
+  profile()->GetPrefs()->SetBoolean(prefs::kExtensionsUIDeveloperMode, true);
+  base::HistogramTester histograms;
+  const char* histogram_name = "Extensions.Debugger.UserIsInDeveloperMode";
+
+  ASSERT_TRUE(RunExtensionTest("debugger_is_developer_mode")) << message_;
+
+  histograms.ExpectBucketCount(histogram_name, true, 1);
+}
+
+IN_PROC_BROWSER_TEST_F(DebuggerExtensionApiTest,
+                       IsDeveloperModeFalseHistogram) {
+  profile()->GetPrefs()->SetBoolean(prefs::kExtensionsUIDeveloperMode, false);
+  base::HistogramTester histograms;
+  const char* histogram_name = "Extensions.Debugger.UserIsInDeveloperMode";
+
+  ASSERT_TRUE(RunExtensionTest("debugger_is_developer_mode")) << message_;
+
+  histograms.ExpectBucketCount(histogram_name, false, 1);
+}
+
 class SitePerProcessDebuggerExtensionApiTest : public DebuggerExtensionApiTest {
  public:
   void SetUpCommandLine(base::CommandLine* command_line) override {
