@@ -5,10 +5,22 @@
 #include "chrome/browser/net/net_error_diagnostics_dialog.h"
 
 #include "ash/constants/ash_features.h"
+#include "ash/webui/connectivity_diagnostics/url_constants.h"
+#include "base/check.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/chrome_pages.h"
 #include "chrome/browser/ui/web_applications/system_web_app_ui_utils.h"
-#include "chromeos/components/connectivity_diagnostics/url_constants.h"
+
+namespace {
+void LaunchDiagnosticsAppAtConnectivityScreen(Profile* profile) {
+  DCHECK(ash::features::IsNetworkingInDiagnosticsAppEnabled());
+  std::string diagnostics_connectivity_url = {
+      "chrome://diagnostics/?connectivity"};
+  web_app::SystemAppLaunchParams params;
+  params.url = GURL(diagnostics_connectivity_url);
+  LaunchSystemWebAppAsync(profile, web_app::SystemAppType::DIAGNOSTICS, params);
+}
+}  // namespace
 
 bool CanShowNetworkDiagnosticsDialog(content::WebContents* web_contents) {
   // The ChromeOS network diagnostics dialog can be shown in incognito and guest
@@ -21,6 +33,10 @@ void ShowNetworkDiagnosticsDialog(content::WebContents* web_contents,
   Profile* profile =
       Profile::FromBrowserContext(web_contents->GetBrowserContext());
 
-  LaunchSystemWebAppAsync(profile,
-                          web_app::SystemAppType::CONNECTIVITY_DIAGNOSTICS);
+  if (ash::features::IsNetworkingInDiagnosticsAppEnabled()) {
+    LaunchDiagnosticsAppAtConnectivityScreen(std::move(profile));
+  } else {
+    LaunchSystemWebAppAsync(profile,
+                            web_app::SystemAppType::CONNECTIVITY_DIAGNOSTICS);
+  }
 }
