@@ -40,15 +40,14 @@ namespace {
 
 const char kReportingHost[] = "example.com";
 
-class ReportingBrowserTest : public CertVerifierBrowserTest,
-                             public ::testing::WithParamInterface<bool> {
+class BaseReportingBrowserTest : public CertVerifierBrowserTest,
+                                 public ::testing::WithParamInterface<bool> {
  public:
-  ReportingBrowserTest()
+  BaseReportingBrowserTest()
       : https_server_(net::test_server::EmbeddedTestServer::TYPE_HTTPS) {
     std::vector<base::Feature> required_features = {
         network::features::kReporting, network::features::kNetworkErrorLogging,
-        features::kCrashReporting,
-        net::features::kPartitionNelAndReportingByNetworkIsolationKey};
+        features::kCrashReporting};
     if (UseDocumentReporting()) {
       required_features.push_back(net::features::kDocumentReporting);
     }
@@ -59,7 +58,10 @@ class ReportingBrowserTest : public CertVerifierBrowserTest,
         {});
   }
 
-  ~ReportingBrowserTest() override = default;
+  BaseReportingBrowserTest(const BaseReportingBrowserTest&) = delete;
+  BaseReportingBrowserTest& operator=(const BaseReportingBrowserTest&) = delete;
+
+  ~BaseReportingBrowserTest() override = default;
 
   void SetUp() override;
   void SetUpOnMainThread() override;
@@ -122,20 +124,18 @@ class ReportingBrowserTest : public CertVerifierBrowserTest,
   std::unique_ptr<net::test_server::ControllableHttpResponse>
       original_response_;
   std::unique_ptr<net::test_server::ControllableHttpResponse> upload_response_;
-
-  DISALLOW_COPY_AND_ASSIGN(ReportingBrowserTest);
 };
 
-void ReportingBrowserTest::SetUp() {
+void BaseReportingBrowserTest::SetUp() {
   CertVerifierBrowserTest::SetUp();
 
   // Make report delivery happen instantly.
   net::ReportingPolicy policy;
-  policy.delivery_interval = base::TimeDelta::FromSeconds(0);
+  policy.delivery_interval = base::Seconds(0);
   net::ReportingPolicy::UsePolicyForTesting(policy);
 }
 
-void ReportingBrowserTest::SetUpOnMainThread() {
+void BaseReportingBrowserTest::SetUpOnMainThread() {
   CertVerifierBrowserTest::SetUpOnMainThread();
 
   host_resolver()->AddRule("*", "127.0.0.1");
@@ -156,10 +156,44 @@ void ReportingBrowserTest::SetUpOnMainThread() {
   ASSERT_TRUE(server()->Start());
 }
 
+class ReportingBrowserTest : public BaseReportingBrowserTest {
+ public:
+  ReportingBrowserTest() {
+    scoped_feature_list_.InitAndEnableFeature(
+        net::features::kPartitionNelAndReportingByNetworkIsolationKey);
+  }
+
+  ReportingBrowserTest(const ReportingBrowserTest&) = delete;
+  ReportingBrowserTest& operator=(const ReportingBrowserTest&) = delete;
+
+  ~ReportingBrowserTest() override = default;
+
+ private:
+  base::test::ScopedFeatureList scoped_feature_list_;
+};
+
+class NonIsolatedReportingBrowserTest : public BaseReportingBrowserTest {
+ public:
+  NonIsolatedReportingBrowserTest() {
+    scoped_feature_list_.InitAndDisableFeature(
+        net::features::kPartitionNelAndReportingByNetworkIsolationKey);
+  }
+
+  NonIsolatedReportingBrowserTest(const NonIsolatedReportingBrowserTest&) =
+      delete;
+  NonIsolatedReportingBrowserTest& operator=(
+      const NonIsolatedReportingBrowserTest&) = delete;
+
+  ~NonIsolatedReportingBrowserTest() override = default;
+
+ private:
+  base::test::ScopedFeatureList scoped_feature_list_;
+};
+
 std::unique_ptr<base::Value> ParseReportUpload(const std::string& payload) {
   auto parsed_payload = base::test::ParseJsonDeprecated(payload);
   // Clear out any non-reproducible fields.
-  for (auto& report : parsed_payload->GetList()) {
+  for (auto& report : parsed_payload->GetListDeprecated()) {
     report.RemoveKey("age");
     report.RemovePath("body.elapsed_time");
     auto* user_agent =
@@ -272,6 +306,62 @@ IN_PROC_BROWSER_TEST_P(ReportingBrowserTest, TestReportingHeadersProcessed) {
   EXPECT_EQ(expected, *actual);
 }
 
+// Tests that CSP reports are delivered properly whether configured with the
+// v0 Report-To header or the v1 Reporting-Endpoints header. This is a Non-
+// isolated test, so will run with NIK-based report isolation disabled. This is
+// a regression test for https://crbug.com/1258112.
+IN_PROC_BROWSER_TEST_P(NonIsolatedReportingBrowserTest,
+                       TestReportingHeadersProcessed) {
+  NavigateParams params(browser(), GetReportingEnabledURL(),
+                        ui::PAGE_TRANSITION_LINK);
+  Navigate(&params);
+
+  original_response()->WaitForRequest();
+  original_response()->Send("HTTP/1.1 200 OK\r\n");
+  original_response()->Send("Content-Type: text/html\r\n");
+  original_response()->Send(GetAppropriateReportingHeader());
+  original_response()->Send(GetCSPHeader());
+  original_response()->Send("\r\n");
+  original_response()->Send("<script>alert(1);</script>\r\n");
+  original_response()->Done();
+
+  // Ensure that the correct endpoint was found, and that a report was sent.
+  // (If the endpoint cannot not be found, then a report will be sent at all.)
+  upload_response()->WaitForRequest();
+  std::unique_ptr<base::Value> actual =
+      ParseReportUpload(upload_response()->http_request()->content);
+  upload_response()->Send("HTTP/1.1 204 OK\r\n");
+  upload_response()->Send("\r\n");
+  upload_response()->Done();
+
+  // Verify the contents of the report that we received.
+  ASSERT_TRUE(actual);
+  base::Value expected = base::test::ParseJson(base::StringPrintf(
+      R"json(
+        [ {
+           "body": {
+              "blockedURL": "inline",
+              "disposition": "enforce",
+              "documentURL": "%s",
+              "effectiveDirective": "script-src-elem",
+              "lineNumber": 1,
+              "originalPolicy": "script-src 'none'; report-to default",
+              "referrer": "",
+              "sample": "",
+              "sourceFile": "%s",
+              "statusCode": 200
+           },
+           "type": "csp-violation",
+           "url": "%s",
+           "user_agent": "Mozilla/1.0"
+        } ]
+      )json",
+      GetReportingEnabledURL().spec().c_str(),
+      GetReportingEnabledURL().spec().c_str(),
+      GetReportingEnabledURL().spec().c_str()));
+  EXPECT_EQ(expected, *actual);
+}
+
 IN_PROC_BROWSER_TEST_P(ReportingBrowserTest,
                        ReportingRespectsNetworkIsolationKeys) {
   // Navigate main frame to a kReportingHost URL and learn NEL and Reporting
@@ -292,8 +382,8 @@ IN_PROC_BROWSER_TEST_P(ReportingBrowserTest,
 
   // Open a cross-origin kReportingHost iframe that fails to load. No report
   // should be uploaded, since the NetworkIsolationKey does not match.
-  ui_test_utils::NavigateToURL(browser(),
-                               server()->GetURL("/iframe_blank.html"));
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(
+      browser(), server()->GetURL("/iframe_blank.html")));
   content::NavigateIframeToURL(
       browser()->tab_strip_model()->GetActiveWebContents(), "test",
       server()->GetURL(kReportingHost, "/close-socket?should-not-be-reported"));
@@ -303,7 +393,7 @@ IN_PROC_BROWSER_TEST_P(ReportingBrowserTest,
   // the original request where reporting information was learned.
   GURL expect_reported_url =
       server()->GetURL(kReportingHost, "/close-socket?should-be-reported");
-  ui_test_utils::NavigateToURL(browser(), expect_reported_url);
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(browser(), expect_reported_url));
   upload_response()->WaitForRequest();
   std::unique_ptr<base::Value> actual =
       ParseReportUpload(upload_response()->http_request()->content);
@@ -341,8 +431,14 @@ IN_PROC_BROWSER_TEST_P(ReportingBrowserTest,
 #define MAYBE_CrashReportUnresponsive DISABLED_CrashReportUnresponsive
 #else
 #define MAYBE_CrashReport CrashReport
+
+// Flaky on Mac (multiple versions), see https://crbug.com/1261749
+#if BUILDFLAG(IS_MAC)
+#define MAYBE_CrashReportUnresponsive DISABLED_CrashReportUnresponsive
+#else
 #define MAYBE_CrashReportUnresponsive CrashReportUnresponsive
 #endif
+#endif
 
 IN_PROC_BROWSER_TEST_P(ReportingBrowserTest, MAYBE_CrashReport) {
   content::WebContents* contents =
@@ -375,7 +471,7 @@ IN_PROC_BROWSER_TEST_P(ReportingBrowserTest, MAYBE_CrashReport) {
 
   // Verify the contents of the report that we received.
   EXPECT_TRUE(response != nullptr);
-  auto report = response->GetList().begin();
+  auto report = response->GetListDeprecated().begin();
   auto* type = report->FindKeyOfType("type", base::Value::Type::STRING);
   auto* url = report->FindKeyOfType("url", base::Value::Type::STRING);
 
@@ -412,7 +508,7 @@ IN_PROC_BROWSER_TEST_P(ReportingBrowserTest, MAYBE_CrashReportUnresponsive) {
 
   // Verify the contents of the report that we received.
   EXPECT_TRUE(response != nullptr);
-  auto report = response->GetList().begin();
+  auto report = response->GetListDeprecated().begin();
   auto* type = report->FindKeyOfType("type", base::Value::Type::STRING);
   auto* url = report->FindKeyOfType("url", base::Value::Type::STRING);
   auto* body = report->FindKeyOfType("body", base::Value::Type::DICTIONARY);
@@ -424,3 +520,6 @@ IN_PROC_BROWSER_TEST_P(ReportingBrowserTest, MAYBE_CrashReportUnresponsive) {
 }
 
 INSTANTIATE_TEST_SUITE_P(All, ReportingBrowserTest, ::testing::Bool());
+INSTANTIATE_TEST_SUITE_P(All,
+                         NonIsolatedReportingBrowserTest,
+                         ::testing::Bool());
