@@ -5,7 +5,7 @@
 #ifndef CHROME_BROWSER_UI_WEBUI_CHROMEOS_CELLULAR_SETUP_MOBILE_SETUP_DIALOG_H_
 #define CHROME_BROWSER_UI_WEBUI_CHROMEOS_CELLULAR_SETUP_MOBILE_SETUP_DIALOG_H_
 
-#include "base/macros.h"
+#include "chrome/browser/ui/ash/network/network_connect_delegate_chromeos.h"
 #include "chrome/browser/ui/webui/chromeos/system_web_dialog_delegate.h"
 
 namespace chromeos {
@@ -14,10 +14,12 @@ class NetworkState;
 
 namespace cellular_setup {
 
-// Dialog used for cellular activation flow when the
-// kUpdatedCellularActivationUi flag is disabled.
 // DEPRECATED: Being replaced by new UI; see https://crbug.com/778021.
 class MobileSetupDialog : public SystemWebDialogDelegate {
+ public:
+  MobileSetupDialog(const MobileSetupDialog&) = delete;
+  MobileSetupDialog& operator=(const MobileSetupDialog&) = delete;
+
  protected:
   explicit MobileSetupDialog(const NetworkState& network);
   ~MobileSetupDialog() override;
@@ -29,10 +31,9 @@ class MobileSetupDialog : public SystemWebDialogDelegate {
                        bool* out_close_dialog) override;
 
  private:
-  friend void OpenCellularSetupDialog(const std::string& cellular_network_guid);
+  friend void NetworkConnectDelegateChromeOS::ShowCarrierAccountDetail(
+      const std::string& network_id);
   static void ShowByNetworkId(const std::string& network_id);
-
-  DISALLOW_COPY_AND_ASSIGN(MobileSetupDialog);
 };
 
 }  // namespace cellular_setup
