@@ -8,11 +8,13 @@
 
 #include "base/bind.h"
 #include "base/callback.h"
+#include "base/memory/raw_ptr.h"
 #include "base/test/bind.h"
 #include "base/test/gmock_callback_support.h"
 #include "base/test/mock_callback.h"
 #include "base/test/task_environment.h"
 #include "base/test/test_simple_task_runner.h"
+#include "base/time/time.h"
 #include "base/timer/timer.h"
 #include "components/autofill_assistant/browser/actions/mock_action_delegate.h"
 #include "components/autofill_assistant/browser/wait_for_dom_observer.h"
@@ -42,10 +44,12 @@ class PromptActionTest : public testing::Test {
       : task_env_(base::test::TaskEnvironment::TimeSource::MOCK_TIME) {}
 
   void SetUp() override {
-    ON_CALL(mock_web_controller_, OnFindElement(_, _))
-        .WillByDefault(RunOnceCallback<1>(
-            ClientStatus(ELEMENT_RESOLUTION_FAILED), nullptr));
-    EXPECT_CALL(mock_action_delegate_, WaitForDom(_, _, _, _, _))
+    ON_CALL(mock_web_controller_, FindElement(_, _, _))
+        .WillByDefault(WithArgs<2>([](auto&& callback) {
+          std::move(callback).Run(ClientStatus(ELEMENT_RESOLUTION_FAILED),
+                                  std::make_unique<ElementFinderResult>());
+        }));
+    EXPECT_CALL(mock_action_delegate_, WaitForDom)
         .WillRepeatedly(Invoke(this, &PromptActionTest::FakeWaitForDom));
     ON_CALL(mock_action_delegate_, Prompt(_, _, _, _, _))
         .WillByDefault(
@@ -65,6 +69,7 @@ class PromptActionTest : public testing::Test {
   // until it gets a successful callback, then calls done_waiting_callback.
   void FakeWaitForDom(
       base::TimeDelta max_wait_time,
+      bool allow_observer_mode,
       bool allow_interrupt,
       WaitForDomObserver* observer,
       base::RepeatingCallback<
@@ -88,8 +93,7 @@ class PromptActionTest : public testing::Test {
     check_elements.Run(checker_.get(),
                        base::BindOnce(&PromptActionTest::OnCheckElementsDone,
                                       base::Unretained(this)));
-    task_env_.FastForwardBy(
-        base::TimeDelta::FromMilliseconds(fake_check_time_));
+    task_env_.FastForwardBy(base::Milliseconds(fake_check_time_));
     checker_->AddAllDoneCallback(
         base::BindOnce(&PromptActionTest::OnWaitForDomDone,
                        base::Unretained(this), check_elements));
@@ -117,12 +121,11 @@ class PromptActionTest : public testing::Test {
 
     if (check_elements_result_.ok()) {
       std::move(fake_wait_for_dom_done_)
-          .Run(check_elements_result_,
-               base::TimeDelta::FromMilliseconds(fake_wait_time_));
+          .Run(check_elements_result_, base::Milliseconds(fake_wait_time_));
     } else {
       wait_for_dom_timer_ = std::make_unique<base::OneShotTimer>();
       wait_for_dom_timer_->Start(
-          FROM_HERE, base::TimeDelta::FromSeconds(1),
+          FROM_HERE, base::Seconds(1),
           base::BindOnce(&PromptActionTest::RunFakeWaitForDom,
                          base::Unretained(this), check_elements));
     }
@@ -138,7 +141,7 @@ class PromptActionTest : public testing::Test {
   base::OnceCallback<void(const ClientStatus&, base::TimeDelta)>
       fake_wait_for_dom_done_;
   ActionProto proto_;
-  PromptProto* prompt_proto_;
+  raw_ptr<PromptProto> prompt_proto_;
   std::unique_ptr<std::vector<UserAction>> user_actions_;
   std::unique_ptr<BatchElementChecker> checker_;
   bool has_check_elements_result_ = false;
@@ -162,6 +165,7 @@ TEST_F(PromptActionTest, SelectButtons) {
   chip->set_text("Ok");
   chip->set_type(HIGHLIGHTED_ACTION);
   ok_proto->set_server_payload("ok");
+  ok_proto->set_tag("oktag");
 
   auto* cancel_proto = prompt_proto_->add_choices();
   cancel_proto->mutable_chip()->set_text("Cancel");
@@ -186,34 +190,11 @@ TEST_F(PromptActionTest, SelectButtons) {
           Property(&ProcessedActionProto::prompt_choice,
                    Property(&PromptProto::Result::navigation_ended, false)),
           Property(&ProcessedActionProto::prompt_choice,
-                   Property(&PromptProto::Result::server_payload, "ok"))))));
+                   Property(&PromptProto::Result::server_payload, "ok")),
+          Property(&ProcessedActionProto::prompt_choice,
+                   Property(&PromptProto::Result::choice_tag, "oktag"))))));
   EXPECT_TRUE((*user_actions_)[0].HasCallback());
-  (*user_actions_)[0].Call(std::make_unique<TriggerContext>());
-}
-
-TEST_F(PromptActionTest, ReportDirectAction) {
-  // Ok has a chip and a direct action.
-  auto* ok_proto = prompt_proto_->add_choices();
-  ok_proto->mutable_chip()->set_text("Ok");
-  ok_proto->mutable_direct_action()->add_names("ok");
-  ok_proto->set_server_payload("ok");
-
-  // Maybe only has a mappings to direct actions.
-  auto* maybe_proto = prompt_proto_->add_choices();
-  maybe_proto->mutable_direct_action()->add_names("maybe");
-  maybe_proto->mutable_direct_action()->add_names("I_guess");
-  maybe_proto->set_server_payload("maybe");
-
-  PromptAction action(&mock_action_delegate_, proto_);
-  action.ProcessAction(callback_.Get());
-
-  ASSERT_THAT(user_actions_, Pointee(SizeIs(2)));
-
-  EXPECT_THAT((*user_actions_)[0].direct_action().names, ElementsAre("ok"));
-  EXPECT_FALSE((*user_actions_)[0].chip().empty());
-  EXPECT_THAT((*user_actions_)[1].direct_action().names,
-              UnorderedElementsAre("maybe", "I_guess"));
-  EXPECT_TRUE((*user_actions_)[1].chip().empty());
+  (*user_actions_)[0].RunCallback();
 }
 
 TEST_F(PromptActionTest, ShowOnlyIfElementExists) {
@@ -229,18 +210,20 @@ TEST_F(PromptActionTest, ShowOnlyIfElementExists) {
 
   ASSERT_THAT(user_actions_, Pointee(IsEmpty()));
 
-  EXPECT_CALL(mock_web_controller_, OnFindElement(Selector({"element"}), _))
-      .WillRepeatedly(WithArgs<1>([](auto&& callback) {
+  EXPECT_CALL(mock_web_controller_, FindElement(Selector({"element"}), _, _))
+      .WillRepeatedly(WithArgs<2>([](auto&& callback) {
         std::move(callback).Run(OkClientStatus(),
-                                std::make_unique<ElementFinder::Result>());
+                                std::make_unique<ElementFinderResult>());
       }));
-  task_env_.FastForwardBy(base::TimeDelta::FromSeconds(1));
+  task_env_.FastForwardBy(base::Seconds(1));
   ASSERT_THAT(user_actions_, Pointee(SizeIs(1)));
 
-  EXPECT_CALL(mock_web_controller_, OnFindElement(Selector({"element"}), _))
-      .WillRepeatedly(
-          RunOnceCallback<1>(ClientStatus(ELEMENT_RESOLUTION_FAILED), nullptr));
-  task_env_.FastForwardBy(base::TimeDelta::FromSeconds(1));
+  EXPECT_CALL(mock_web_controller_, FindElement(Selector({"element"}), _, _))
+      .WillRepeatedly(WithArgs<2>([](auto&& callback) {
+        std::move(callback).Run(ClientStatus(ELEMENT_RESOLUTION_FAILED),
+                                std::make_unique<ElementFinderResult>());
+      }));
+  task_env_.FastForwardBy(base::Seconds(1));
   ASSERT_THAT(user_actions_, Pointee(IsEmpty()));
 }
 
@@ -252,31 +235,31 @@ TEST_F(PromptActionTest, TimingStatsUserAction) {
   *ok_proto->mutable_show_only_when()->mutable_match() =
       ToSelectorProto("element");
 
-  EXPECT_CALL(mock_web_controller_, OnFindElement(Selector({"element"}), _))
-      .WillOnce(WithArgs<1>([](auto&& callback) {
+  EXPECT_CALL(mock_web_controller_, FindElement(Selector({"element"}), _, _))
+      .WillOnce(WithArgs<2>([](auto&& callback) {
         std::move(callback).Run(ClientStatus(ELEMENT_RESOLUTION_FAILED),
-                                std::make_unique<ElementFinder::Result>());
+                                std::make_unique<ElementFinderResult>());
       }))
-      .WillOnce(WithArgs<1>([](auto&& callback) {
+      .WillOnce(WithArgs<2>([](auto&& callback) {
         std::move(callback).Run(ClientStatus(ELEMENT_RESOLUTION_FAILED),
-                                std::make_unique<ElementFinder::Result>());
+                                std::make_unique<ElementFinderResult>());
       }))
-      .WillRepeatedly(WithArgs<1>([](auto&& callback) {
+      .WillRepeatedly(WithArgs<2>([](auto&& callback) {
         std::move(callback).Run(OkClientStatus(),
-                                std::make_unique<ElementFinder::Result>());
+                                std::make_unique<ElementFinderResult>());
       }));
 
   fake_check_time_ = 200;
   PromptAction action(&mock_action_delegate_, proto_);
   action.ProcessAction(callback_.Get());
 
-  task_env_.FastForwardBy(base::TimeDelta::FromSeconds(3));
+  task_env_.FastForwardBy(base::Seconds(3));
   ASSERT_THAT(user_actions_, Pointee(SizeIs(1)));
 
   ProcessedActionProto capture;
   EXPECT_CALL(callback_, Run(_)).WillOnce(SaveArgPointee<0>(&capture));
   EXPECT_TRUE((*user_actions_)[0].HasCallback());
-  (*user_actions_)[0].Call(std::make_unique<TriggerContext>());
+  (*user_actions_)[0].RunCallback();
   EXPECT_EQ(capture.timing_stats().active_time_ms(), 700);
   EXPECT_EQ(capture.timing_stats().wait_time_ms(), 2500);
 }
@@ -293,19 +276,21 @@ TEST_F(PromptActionTest, DisabledUnlessElementExists) {
   PromptAction action(&mock_action_delegate_, proto_);
   action.ProcessAction(callback_.Get());
 
-  EXPECT_CALL(mock_web_controller_, OnFindElement(Selector({"element"}), _))
-      .WillRepeatedly(WithArgs<1>([](auto&& callback) {
+  EXPECT_CALL(mock_web_controller_, FindElement(Selector({"element"}), _, _))
+      .WillRepeatedly(WithArgs<2>([](auto&& callback) {
         std::move(callback).Run(OkClientStatus(),
-                                std::make_unique<ElementFinder::Result>());
+                                std::make_unique<ElementFinderResult>());
       }));
-  task_env_.FastForwardBy(base::TimeDelta::FromSeconds(1));
+  task_env_.FastForwardBy(base::Seconds(1));
   ASSERT_THAT(user_actions_, Pointee(SizeIs(1)));
   EXPECT_TRUE((*user_actions_)[0].enabled());
 
-  EXPECT_CALL(mock_web_controller_, OnFindElement(Selector({"element"}), _))
-      .WillRepeatedly(
-          RunOnceCallback<1>(ClientStatus(ELEMENT_RESOLUTION_FAILED), nullptr));
-  task_env_.FastForwardBy(base::TimeDelta::FromSeconds(1));
+  EXPECT_CALL(mock_web_controller_, FindElement(Selector({"element"}), _, _))
+      .WillRepeatedly(WithArgs<2>([](auto&& callback) {
+        std::move(callback).Run(ClientStatus(ELEMENT_RESOLUTION_FAILED),
+                                std::make_unique<ElementFinderResult>());
+      }));
+  task_env_.FastForwardBy(base::Seconds(1));
   ASSERT_THAT(user_actions_, Pointee(SizeIs(1)));
   EXPECT_FALSE((*user_actions_)[0].enabled());
   EXPECT_TRUE((*user_actions_)[0].HasCallback());
@@ -321,10 +306,10 @@ TEST_F(PromptActionTest, AutoSelectWhenElementExists) {
   action.ProcessAction(callback_.Get());
   EXPECT_THAT(user_actions_, Pointee(SizeIs(0)));
 
-  EXPECT_CALL(mock_web_controller_, OnFindElement(Selector({"element"}), _))
-      .WillRepeatedly(WithArgs<1>([](auto&& callback) {
+  EXPECT_CALL(mock_web_controller_, FindElement(Selector({"element"}), _, _))
+      .WillRepeatedly(WithArgs<2>([](auto&& callback) {
         std::move(callback).Run(OkClientStatus(),
-                                std::make_unique<ElementFinder::Result>());
+                                std::make_unique<ElementFinderResult>());
       }));
 
   EXPECT_CALL(mock_action_delegate_, CleanUpAfterPrompt());
@@ -334,7 +319,7 @@ TEST_F(PromptActionTest, AutoSelectWhenElementExists) {
                         Property(&ProcessedActionProto::prompt_choice,
                                  Property(&PromptProto::Result::server_payload,
                                           "auto-select"))))));
-  task_env_.FastForwardBy(base::TimeDelta::FromSeconds(1));
+  task_env_.FastForwardBy(base::Seconds(1));
 }
 
 TEST_F(PromptActionTest, TimingStatsAutoSelect) {
@@ -348,16 +333,16 @@ TEST_F(PromptActionTest, TimingStatsAutoSelect) {
   action.ProcessAction(callback_.Get());
   EXPECT_THAT(user_actions_, Pointee(SizeIs(0)));
 
-  EXPECT_CALL(mock_web_controller_, OnFindElement(Selector({"element"}), _))
-      .WillRepeatedly(WithArgs<1>([](auto&& callback) {
+  EXPECT_CALL(mock_web_controller_, FindElement(Selector({"element"}), _, _))
+      .WillRepeatedly(WithArgs<2>([](auto&& callback) {
         std::move(callback).Run(OkClientStatus(),
-                                std::make_unique<ElementFinder::Result>());
+                                std::make_unique<ElementFinderResult>());
       }));
 
   EXPECT_CALL(mock_action_delegate_, CleanUpAfterPrompt());
   ProcessedActionProto capture;
   EXPECT_CALL(callback_, Run(_)).WillOnce(SaveArgPointee<0>(&capture));
-  task_env_.FastForwardBy(base::TimeDelta::FromSeconds(1));
+  task_env_.FastForwardBy(base::Seconds(1));
   EXPECT_EQ(capture.timing_stats().active_time_ms(), 500);
   EXPECT_EQ(capture.timing_stats().wait_time_ms(), 500);
 }
@@ -378,10 +363,10 @@ TEST_F(PromptActionTest, AutoSelectWithButton) {
 
   ASSERT_THAT(user_actions_, Pointee(SizeIs(1)));
 
-  EXPECT_CALL(mock_web_controller_, OnFindElement(Selector({"element"}), _))
-      .WillRepeatedly(WithArgs<1>([](auto&& callback) {
+  EXPECT_CALL(mock_web_controller_, FindElement(Selector({"element"}), _, _))
+      .WillRepeatedly(WithArgs<2>([](auto&& callback) {
         std::move(callback).Run(OkClientStatus(),
-                                std::make_unique<ElementFinder::Result>());
+                                std::make_unique<ElementFinderResult>());
       }));
   EXPECT_CALL(
       callback_,
@@ -389,7 +374,7 @@ TEST_F(PromptActionTest, AutoSelectWithButton) {
                         Property(&ProcessedActionProto::prompt_choice,
                                  Property(&PromptProto::Result::server_payload,
                                           "auto-select"))))));
-  task_env_.FastForwardBy(base::TimeDelta::FromSeconds(1));
+  task_env_.FastForwardBy(base::Seconds(1));
 }
 
 TEST_F(PromptActionTest, Terminate) {
@@ -405,7 +390,7 @@ TEST_F(PromptActionTest, Terminate) {
   // Chips pointing to a deleted action do nothing.
   ASSERT_THAT(user_actions_, Pointee(SizeIs(1)));
   EXPECT_TRUE((*user_actions_)[0].HasCallback());
-  (*user_actions_)[0].Call(std::make_unique<TriggerContext>());
+  (*user_actions_)[0].RunCallback();
 }
 
 TEST_F(PromptActionTest, NoMessageSet) {
@@ -506,7 +491,7 @@ TEST_F(PromptActionTest, ForwardInterruptFailure) {
   EXPECT_THAT(user_actions_, Pointee(SizeIs(0)));
 
   // First round of element checks: element doesn't exist.
-  task_env_.FastForwardBy(base::TimeDelta::FromSeconds(1));
+  task_env_.FastForwardBy(base::Seconds(1));
 
   // Second round of element checks: an interrupt ran and failed. No choice was
   // selected.
@@ -519,7 +504,7 @@ TEST_F(PromptActionTest, ForwardInterruptFailure) {
                        Property(&PromptProto::Result::server_payload, ""))))));
   ASSERT_TRUE(fake_wait_for_dom_done_);
   std::move(fake_wait_for_dom_done_)
-      .Run(ClientStatus(INTERRUPT_FAILED), base::TimeDelta::FromSeconds(0));
+      .Run(ClientStatus(INTERRUPT_FAILED), base::Seconds(0));
 }
 
 TEST_F(PromptActionTest, EndActionOnNavigation) {
@@ -558,8 +543,7 @@ TEST_F(PromptActionTest, TimingStatsEndActionOnNavigation) {
                          base::OnceCallback<void()> callback, bool browse_mode,
                          bool browse_mode_invisible) {
             user_actions_ = std::move(user_actions);
-            timer->Start(FROM_HERE, base::TimeDelta::FromSeconds(1),
-                         std::move(callback));
+            timer->Start(FROM_HERE, base::Seconds(1), std::move(callback));
           });
 
   prompt_proto_->set_end_on_navigation(true);
