@@ -6,7 +6,6 @@
 #define COMPONENTS_AUTOFILL_ASSISTANT_BROWSER_SCRIPT_EXECUTOR_H_
 
 #include <deque>
-#include <map>
 #include <memory>
 #include <ostream>
 #include <set>
@@ -14,30 +13,39 @@
 #include <vector>
 
 #include "base/callback_forward.h"
+#include "base/containers/flat_map.h"
+#include "base/memory/raw_ptr.h"
 #include "base/memory/weak_ptr.h"
+#include "base/time/time.h"
 #include "components/autofill_assistant/browser/actions/action.h"
 #include "components/autofill_assistant/browser/actions/action_delegate.h"
 #include "components/autofill_assistant/browser/client_settings.h"
 #include "components/autofill_assistant/browser/details.h"
+#include "components/autofill_assistant/browser/full_card_requester.h"
 #include "components/autofill_assistant/browser/info_box.h"
 #include "components/autofill_assistant/browser/retry_timer.h"
 #include "components/autofill_assistant/browser/script.h"
 #include "components/autofill_assistant/browser/script_executor_delegate.h"
+#include "components/autofill_assistant/browser/script_executor_ui_delegate.h"
 #include "components/autofill_assistant/browser/service.pb.h"
+#include "components/autofill_assistant/browser/service/service_request_sender.h"
 #include "components/autofill_assistant/browser/top_padding.h"
+#include "components/autofill_assistant/browser/user_data.h"
 #include "components/autofill_assistant/browser/wait_for_dom_observer.h"
+#include "components/autofill_assistant/browser/web/element.h"
 #include "components/autofill_assistant/browser/web/element_finder.h"
+#include "components/autofill_assistant/browser/web/element_store.h"
+#include "services/metrics/public/cpp/ukm_recorder.h"
 
 namespace autofill_assistant {
 class ElementStore;
 class UserModel;
-class WaitForDocumentOperation;
+class WaitForDomOperation;
 class WebController;
 
 // Class to execute an assistant script.
 class ScriptExecutor : public ActionDelegate,
-                       public ScriptExecutorDelegate::NavigationListener,
-                       public ScriptExecutorDelegate::Listener {
+                       public ScriptExecutorDelegate::NavigationListener {
  public:
   // Listens to events on ScriptExecutor.
   // TODO(b/806868): Make global_payload a part of callback instead of the
@@ -65,9 +73,13 @@ class ScriptExecutor : public ActionDelegate,
                  const std::string& global_payload,
                  const std::string& script_payload,
                  ScriptExecutor::Listener* listener,
-                 std::map<std::string, ScriptStatusProto>* scripts_state,
                  const std::vector<std::unique_ptr<Script>>* ordered_interrupts,
-                 ScriptExecutorDelegate* delegate);
+                 ScriptExecutorDelegate* delegate,
+                 ScriptExecutorUiDelegate* ui_delegate);
+
+  ScriptExecutor(const ScriptExecutor&) = delete;
+  ScriptExecutor& operator=(const ScriptExecutor&) = delete;
+
   ~ScriptExecutor() override;
 
   // What should happen after the script has run.
@@ -101,15 +113,11 @@ class ScriptExecutor : public ActionDelegate,
   void Run(const UserData* user_data, RunScriptCallback callback);
 
   const UserData* GetUserData() const override;
-  UserModel* GetUserModel() override;
+  UserModel* GetUserModel() const override;
 
   // Override ScriptExecutorDelegate::NavigationListener
   void OnNavigationStateChanged() override;
 
-  // Override ScriptExecutorDelegate::Listener
-  void OnPause(const std::string& message,
-               const std::string& button_label) override;
-
   // Override ActionDelegate:
   void RunElementChecks(BatchElementChecker* checker) override;
   void ShortWaitForElement(
@@ -122,6 +130,7 @@ class ScriptExecutor : public ActionDelegate,
       override;
   void WaitForDom(
       base::TimeDelta max_wait_time,
+      bool allow_observer_mode,
       bool allow_interrupt,
       WaitForDomObserver* observer,
       base::RepeatingCallback<
@@ -139,23 +148,16 @@ class ScriptExecutor : public ActionDelegate,
       base::OnceCallback<void(const ClientStatus&, base::TimeDelta)> callback)
       override;
   void SetStatusMessage(const std::string& message) override;
-  std::string GetStatusMessage() override;
+  std::string GetStatusMessage() const override;
   void SetBubbleMessage(const std::string& message) override;
-  std::string GetBubbleMessage() override;
+  std::string GetBubbleMessage() const override;
+  void SetTtsMessage(const std::string& message) override;
+  TtsButtonState GetTtsButtonState() const override;
+  void MaybePlayTtsMessage() override;
   void FindElement(const Selector& selector,
                    ElementFinder::Callback callback) const override;
   void FindAllElements(const Selector& selector,
                        ElementFinder::Callback callback) const override;
-  void WaitUntilElementIsStable(
-      int max_rounds,
-      base::TimeDelta check_interval,
-      const ElementFinder::Result& element,
-      base::OnceCallback<void(const ClientStatus&, base::TimeDelta)> callback)
-      override;
-  void ClickOrTapElement(
-      ClickType click_type,
-      const ElementFinder::Result& element,
-      base::OnceCallback<void(const ClientStatus&)> callback) override;
   void CollectUserData(
       CollectUserDataOptions* collect_user_data_options) override;
   void SetLastSuccessfulUserDataOptions(std::unique_ptr<CollectUserDataOptions>
@@ -163,7 +165,7 @@ class ScriptExecutor : public ActionDelegate,
   const CollectUserDataOptions* GetLastSuccessfulUserDataOptions()
       const override;
   void WriteUserData(
-      base::OnceCallback<void(UserData*, UserData::FieldChange*)>) override;
+      base::OnceCallback<void(UserData*, UserDataFieldChange*)>) override;
   void GetFullCard(const autofill::CreditCard* credit_card,
                    GetFullCardCallback callback) override;
   void Prompt(std::unique_ptr<std::vector<UserAction>> user_actions,
@@ -173,92 +175,50 @@ class ScriptExecutor : public ActionDelegate,
               bool browse_mode_invisible) override;
   void CleanUpAfterPrompt() override;
   void SetBrowseDomainsAllowlist(std::vector<std::string> domains) override;
-  void FillAddressForm(
-      const autofill::AutofillProfile* profile,
-      const Selector& selector,
-      base::OnceCallback<void(const ClientStatus&)> callback) override;
-  void FillCardForm(
-      std::unique_ptr<autofill::CreditCard> card,
-      const base::string16& cvc,
-      const Selector& selector,
-      base::OnceCallback<void(const ClientStatus&)> callback) override;
   void RetrieveElementFormAndFieldData(
       const Selector& selector,
       base::OnceCallback<void(const ClientStatus&,
                               const autofill::FormData& form_data,
                               const autofill::FormFieldData& field_data)>
           callback) override;
-  void SelectOption(
-      const std::string& re2,
-      bool case_sensitive,
-      SelectOptionProto::OptionComparisonAttribute option_comparison_attribute,
-      const ElementFinder::Result& element,
-      base::OnceCallback<void(const ClientStatus&)> callback) override;
-  void ScrollToElementPosition(
-      const Selector& selector,
-      const TopPadding& top_padding,
-      std::unique_ptr<ElementFinder::Result> container,
-      const ElementFinder::Result& element,
-      base::OnceCallback<void(const ClientStatus&)> callback) override;
+  void StoreScrolledToElement(const ElementFinderResult& element) override;
   void SetTouchableElementArea(
       const ElementAreaProto& touchable_element_area) override;
-  void GetFieldValue(
-      const ElementFinder::Result& element,
-      base::OnceCallback<void(const ClientStatus&, const std::string&)>
-          callback) override;
-  void GetStringAttribute(
-      const std::vector<std::string>& attributes,
-      const ElementFinder::Result& element,
-      base::OnceCallback<void(const ClientStatus&, const std::string&)>
-          callback) override;
-  void SetValueAttribute(
-      const std::string& value,
-      const ElementFinder::Result& element,
-      base::OnceCallback<void(const ClientStatus&)> callback) override;
-  void SetAttribute(
-      const std::vector<std::string>& attributes,
-      const std::string& value,
-      const ElementFinder::Result& element,
-      base::OnceCallback<void(const ClientStatus&)> callback) override;
-  void SendKeyboardInput(
-      const std::vector<UChar32>& codepoints,
-      int key_press_delay_in_millisecond,
-      const ElementFinder::Result& element,
-      base::OnceCallback<void(const ClientStatus&)> callback) override;
   void ExpectNavigation() override;
   bool ExpectedNavigationHasStarted() override;
   bool WaitForNavigation(base::OnceCallback<void(bool)> callback) override;
   void WaitForDocumentReadyState(
       base::TimeDelta max_wait_time,
       DocumentReadyState min_ready_state,
-      const ElementFinder::Result& optional_frame_element,
+      const ElementFinderResult& optional_frame_element,
       base::OnceCallback<void(const ClientStatus&,
                               DocumentReadyState,
                               base::TimeDelta)> callback) override;
   void WaitUntilDocumentIsInReadyState(
       base::TimeDelta max_wait_time,
       DocumentReadyState min_ready_state,
-      const ElementFinder::Result& optional_frame_element,
+      const ElementFinderResult& optional_frame_element,
       base::OnceCallback<void(const ClientStatus&, base::TimeDelta)> callback)
       override;
 
   void LoadURL(const GURL& url) override;
   void Shutdown(bool show_feedback_chip) override;
   void Close() override;
-  autofill::PersonalDataManager* GetPersonalDataManager() override;
-  WebsiteLoginManager* GetWebsiteLoginManager() override;
-  content::WebContents* GetWebContents() override;
+  autofill::PersonalDataManager* GetPersonalDataManager() const override;
+  WebsiteLoginManager* GetWebsiteLoginManager() const override;
+  password_manager::PasswordChangeSuccessTracker*
+  GetPasswordChangeSuccessTracker() const override;
+  content::WebContents* GetWebContents() const override;
   ElementStore* GetElementStore() const override;
   WebController* GetWebController() const override;
-  std::string GetEmailAddressForAccessTokenAccount() override;
-  std::string GetLocale() override;
+  std::string GetEmailAddressForAccessTokenAccount() const override;
+  ukm::UkmRecorder* GetUkmRecorder() const override;
   void SetDetails(std::unique_ptr<Details> details,
                   base::TimeDelta delay) override;
   void AppendDetails(std::unique_ptr<Details> details,
                      base::TimeDelta delay) override;
   void ClearInfoBox() override;
   void SetInfoBox(const InfoBox& info_box) override;
-  void SetProgress(int progress) override;
   bool SetProgressActiveStepIdentifier(
       const std::string& active_step_identifier) override;
   void SetProgressActiveStep(int active_step) override;
@@ -268,14 +228,15 @@ class ScriptExecutor : public ActionDelegate,
       const ShowProgressBarProto::StepProgressBarConfiguration& configuration)
       override;
   void SetViewportMode(ViewportMode mode) override;
-  ViewportMode GetViewportMode() override;
+  ViewportMode GetViewportMode() const override;
   void SetPeekMode(ConfigureBottomSheetProto::PeekMode peek_mode) override;
-  ConfigureBottomSheetProto::PeekMode GetPeekMode() override;
+  ConfigureBottomSheetProto::PeekMode GetPeekMode() const override;
   void ExpandBottomSheet() override;
   void CollapseBottomSheet() override;
   void WaitForWindowHeightChange(
       base::OnceCallback<void(const ClientStatus&)> callback) override;
   const ClientSettings& GetSettings() const override;
+  void SetClientSettings(const ClientSettingsProto& client_settings) override;
   bool SetForm(
       std::unique_ptr<FormProto> form,
       base::RepeatingCallback<void(const FormProto::Result*)> changed_callback,
@@ -286,163 +247,55 @@ class ScriptExecutor : public ActionDelegate,
       base::OnceCallback<void(const ClientStatus&)> end_action_callback,
       base::OnceCallback<void(const ClientStatus&)>
           view_inflation_finished_callback) override;
+  void SetPersistentGenericUi(
+      std::unique_ptr<GenericUserInterfaceProto> generic_ui,
+      base::OnceCallback<void(const ClientStatus&)>
+          view_inflation_finished_callback) override;
   void ClearGenericUi() override;
+  void ClearPersistentGenericUi() override;
   void SetOverlayBehavior(
       ConfigureUiStateProto::OverlayBehavior overlay_behavior) override;
   void MaybeShowSlowWebsiteWarning(
       base::OnceCallback<void(bool)> callback) override;
   void MaybeShowSlowConnectionWarning() override;
-  void DispatchJsEvent(
-      base::OnceCallback<void(const ClientStatus&)> callback) const override;
   base::WeakPtr<ActionDelegate> GetWeakPtr() const override;
+  ProcessedActionStatusDetailsProto& GetLogInfo() override;
+  void RequestUserData(
+      const CollectUserDataOptions& options,
+      base::OnceCallback<void(bool, const GetUserDataResponseProto&)> callback)
+      override;
 
  private:
-  // Helper for WaitForElementVisible that keeps track of the state required to
-  // run interrupts while waiting for a specific element.
-  class WaitForDomOperation : public ScriptExecutor::Listener,
-                              ScriptExecutorDelegate::NavigationListener {
-   public:
-    // Let the caller know about either the result of looking for the element or
-    // of an abnormal result from an interrupt.
-    //
-    // If the given result is non-null, it should be forwarded as the result of
-    // the main script.
-    using Callback = base::OnceCallback<void(const ClientStatus&,
-                                             const ScriptExecutor::Result*,
-                                             base::TimeDelta)>;
-
-    using WarningCallback =
-        base::OnceCallback<void(base::OnceCallback<void(bool)>)>;
-
-    // |main_script_| must not be null and outlive this instance.
-    WaitForDomOperation(
-        ScriptExecutor* main_script,
-        ScriptExecutorDelegate* delegate,
-        base::TimeDelta max_wait_time,
-        bool allow_interrupt,
-        WaitForDomObserver* observer,
-        base::RepeatingCallback<
-            void(BatchElementChecker*,
-                 base::OnceCallback<void(const ClientStatus&)>)> check_elements,
-        WaitForDomOperation::Callback callback);
-    ~WaitForDomOperation() override;
-
-    void Run();
-    void Terminate();
-    void SetTimeoutWarningCallback(WarningCallback timeout_warning);
-
-   private:
-    void Start();
-    void Pause();
-    void Continue();
-
-    // Implements ScriptExecutorDelegate::NavigationListener
-    void OnNavigationStateChanged() override;
-
-    // Implements ScriptExecutor::Listener
-    void OnServerPayloadChanged(const std::string& global_payload,
-                                const std::string& script_payload) override;
-    void OnScriptListChanged(
-        std::vector<std::unique_ptr<Script>> scripts) override;
-
-    void RunChecks(
-        base::OnceCallback<void(const ClientStatus&)> report_attempt_result);
-    void OnPreconditionCheckDone(const std::string& interrupt_path,
-                                 bool precondition_match);
-    void OnElementCheckDone(const ClientStatus&);
-    void OnAllChecksDone(
-        base::OnceCallback<void(const ClientStatus&)> report_attempt_result);
-    void RunInterrupt(const std::string& path);
-    void OnInterruptDone(const ScriptExecutor::Result& result);
-    void RunCallback(const ClientStatus& element_status);
-    void RunCallbackWithResult(const ClientStatus& element_status,
-                               const ScriptExecutor::Result* result);
-    void SetSlowWarningStatus(bool was_shown);
-
-    // Saves the current state and sets save_pre_interrupt_state_.
-    void SavePreInterruptState();
-
-    // Restores the UI states as found by SavePreInterruptState.
-    void RestoreStatusMessage();
-
-    // if save_pre_interrupt_state_ is set, attempt to scroll the page back to
-    // the original area.
-    void RestorePreInterruptScroll();
-
-    void TimeoutWarning();
-
-    ScriptExecutor* main_script_;
-    ScriptExecutorDelegate* delegate_;
-    const base::TimeDelta max_wait_time_;
-    const bool allow_interrupt_;
-    WaitForDomObserver* observer_;
-    base::RepeatingCallback<void(BatchElementChecker*,
-                                 base::OnceCallback<void(const ClientStatus&)>)>
-        check_elements_;
-    WaitForDomOperation::Callback callback_;
-    base::OnceCallback<void(base::OnceCallback<void(bool)>)> warning_callback_;
-    std::unique_ptr<base::OneShotTimer> warning_timer_;
-    base::TimeDelta timeout_warning_delay_;
-
-    SlowWarningStatus warning_status_ = NO_WARNING;
-
-    std::unique_ptr<BatchElementChecker> batch_element_checker_;
-
-    // Path of interrupts from |ordered_interrupts_| that have been found
-    // runnable.
-    std::set<std::string> runnable_interrupts_;
-    ClientStatus element_check_result_;
-
-    // An empty vector of interrupts that can be passed to interrupt_executor_
-    // and outlives it. Interrupts must not run interrupts.
-    const std::vector<std::unique_ptr<Script>> no_interrupts_;
-    Stopwatch wait_time_stopwatch_;
-    Stopwatch period_stopwatch_;
-    base::TimeDelta wait_time_total_;
-
-    // The interrupt that's currently running.
-    std::unique_ptr<ScriptExecutor> interrupt_executor_;
-
-    // If true, pre-interrupt state was saved already. This happens just before
-    // the first interrupt.
-    bool saved_pre_interrupt_state_ = false;
-
-    // The status message that was displayed when the interrupt started.
-    std::string pre_interrupt_status_;
-
-    // Paths of the interrupts that were just run. These interrupts are
-    // prevented from firing for one round.
-    std::set<std::string> ran_interrupts_;
-
-    RetryTimer retry_timer_;
-
-    base::WeakPtrFactory<WaitForDomOperation> weak_ptr_factory_{this};
-
-    DISALLOW_COPY_AND_ASSIGN(WaitForDomOperation);
-  };
+  // TODO(b/220079189): remove this friend declaration.
+  friend class WaitForDomOperation;
 
   void OnGetActions(base::TimeTicks start_time,
                     int http_status,
-                    const std::string& response);
-  bool ProcessNextActionResponse(const std::string& response);
+                    const std::string& response,
+                    const ServiceRequestSender::ResponseInfo& response_info);
+  bool ProcessNextActionResponse(
+      const std::string& response,
+      const ServiceRequestSender::ResponseInfo& response_info);
   void ReportPayloadsToListener();
   void ReportScriptsUpdateToListener(
       std::vector<std::unique_ptr<Script>> scripts);
   void RunCallback(bool success);
   void RunCallbackWithResult(const Result& result);
   void ProcessNextAction();
-  void ProcessAction(Action* action);
+  void ProcessCurrentAction();
   void GetNextActions();
   void OnProcessedAction(base::TimeTicks start_time,
                          std::unique_ptr<ProcessedActionProto> action);
-  void CheckElementMatches(
+  void CheckElementConditionMatches(
       const Selector& selector,
       BatchElementChecker* checker,
       base::OnceCallback<void(const ClientStatus&)> callback);
   void CheckElementMatchesCallback(
       base::OnceCallback<void(const ClientStatus&)> callback,
       const ClientStatus& status,
-      const ElementFinder::Result& ignored_element);
+      const std::vector<std::string>& ignored_payloads,
+      const std::vector<std::string>& ignored_tags,
+      const base::flat_map<std::string, DomObjectFrameStack>& ignored_elements);
   void OnShortWaitForElement(
       base::OnceCallback<void(const ClientStatus&, base::TimeDelta)> callback,
       const ClientStatus& element_status,
@@ -467,41 +320,44 @@ class ScriptExecutor : public ActionDelegate,
       int link,
       UserData* user_data,
       const UserModel* user_model);
-  void OnGetFullCard(GetFullCardCallback callback,
+  void OnGetFullCard(std::unique_ptr<FullCardRequester> full_card_requester,
+                     GetFullCardCallback callback,
                      const ClientStatus& status,
                      std::unique_ptr<autofill::CreditCard> card,
-                     const base::string16& cvc);
-  void OnChosen(UserAction::Callback callback,
-                std::unique_ptr<TriggerContext> context);
+                     const std::u16string& cvc);
   void OnWaitForDocumentReadyState(
       base::OnceCallback<void(const ClientStatus&, base::TimeDelta)> callback,
       const ClientStatus& status,
       DocumentReadyState ready_state,
       base::TimeDelta wait_time);
-  void OnResume();
-
-  // Actions that can manipulate the UserActions should be interrupted, such
-  // that they do not overwrite the paused state.
-  bool ShouldInterruptOnPause(const ActionProto& proto);
+  void OnRequestUserData(
+      base::OnceCallback<void(bool, const GetUserDataResponseProto&)> callback,
+      int http_status,
+      const std::string& response,
+      const ServiceRequestSender::ResponseInfo& response_info);
 
   // Maybe shows the message specified in a callout, depending on the current
   // state and client settings.
   bool MaybeShowSlowWarning(const std::string& message, bool enabled);
 
+  // Returns the current ActionData, or nullptr if there is no current action.
+  Action::ActionData* GetCurrentActionData();
+
   const std::string script_path_;
   std::unique_ptr<TriggerContext> additional_context_;
   std::string last_global_payload_;
   const std::string initial_script_payload_;
   std::string last_script_payload_;
-  ScriptExecutor::Listener* const listener_;
-  ScriptExecutorDelegate* const delegate_;
+  const raw_ptr<ScriptExecutor::Listener> listener_;
+  const raw_ptr<ScriptExecutorDelegate> delegate_;
+  const raw_ptr<ScriptExecutorUiDelegate> ui_delegate_;
   // Set of interrupts that might run during wait for dom or prompt action with
   // allow_interrupt. Sorted by priority; an interrupt that appears on the
   // vector first should run first. Note that the content of this vector can
   // change while the script is running, as a result of OnScriptListChanged
   // being called.
-  const std::vector<std::unique_ptr<Script>>* const ordered_interrupts_;
-  std::map<std::string, ScriptStatusProto>* const scripts_state_;
+  const raw_ptr<const std::vector<std::unique_ptr<Script>>> ordered_interrupts_;
+  std::unique_ptr<ElementStore> element_store_;
   RunScriptCallback callback_;
   std::vector<std::unique_ptr<Action>> actions_;
   std::vector<ProcessedActionProto> processed_actions_;
@@ -510,8 +366,7 @@ class ScriptExecutor : public ActionDelegate,
   bool should_clean_contextual_ui_on_finish_ = false;
   ActionProto::ActionInfoCase previous_action_type_ =
       ActionProto::ACTION_INFO_NOT_SET;
-  Selector last_focused_element_selector_;
-  TopPadding last_focused_element_top_padding_;
+  absl::optional<DomObjectFrameStack> last_focused_element_;
   std::unique_ptr<ElementAreaProto> touchable_element_area_;
 
   // Steps towards the requirements for calling |on_expected_navigation_done_|
@@ -532,44 +387,22 @@ class ScriptExecutor : public ActionDelegate,
   // Callback called the next time |expected_navigation_step_| becomes DONE.
   base::OnceCallback<void(bool)> on_expected_navigation_done_;
 
-  // Data only relevant to the currently running action. It is cleared before an
-  // action is run.
-  struct CurrentActionData {
-    CurrentActionData();
-    ~CurrentActionData();
-    CurrentActionData& operator=(CurrentActionData&& other);
+  // Only set while an action is being executed.
+  raw_ptr<Action> current_action_ = nullptr;
 
-    // Navigation information relevant to the current action.
-    NavigationInfoProto navigation_info;
-
-    std::unique_ptr<WaitForDomOperation> wait_for_dom;
-    std::unique_ptr<WaitForDocumentOperation> wait_for_document;
-
-    // Set to true when a direct action was used to trigger a UserAction within
-    // a prompt. This is reported to the backend.
-    bool direct_action = false;
-
-    // This callback is set when a navigation event should terminate an ongoing
-    // prompt action. Only a prompt action will set a valid callback here.
-    base::OnceCallback<void()> end_prompt_on_navigation_callback;
-  };
-  CurrentActionData current_action_data_;
-  base::Optional<size_t> current_action_index_;
-
-  const UserData* user_data_ = nullptr;
-
-  bool is_paused_ = false;
-  std::string last_status_message_;
+  raw_ptr<const UserData> user_data_ = nullptr;
 
   base::TimeTicks batch_start_time_;
   RoundtripTimingStats roundtrip_timing_stats_;
+  RoundtripNetworkStats roundtrip_network_stats_;
 
   bool connection_warning_already_shown_ = false;
   bool website_warning_already_shown_ = false;
   int consecutive_slow_roundtrip_counter_ = 0;
 
+  uint64_t run_id_ = 0;
+
   base::WeakPtrFactory<ScriptExecutor> weak_ptr_factory_{this};
-  DISALLOW_COPY_AND_ASSIGN(ScriptExecutor);
 };
 }  // namespace autofill_assistant
 
