@@ -8,13 +8,13 @@
 #include <vector>
 
 #include "base/containers/flat_map.h"
-#include "components/update_client/update_client.h"
+#include "components/component_updater/component_updater_service.h"
 #include "third_party/abseil-cpp/absl/types/optional.h"
 
 namespace component_updater {
 
-absl::optional<update_client::CrxComponent> GetComponent(
-    const base::flat_map<std::string, update_client::CrxComponent>& components,
+absl::optional<ComponentRegistration> GetComponent(
+    const base::flat_map<std::string, ComponentRegistration>& components,
     const std::string& id) {
   const auto it = components.find(id);
   if (it != components.end())
@@ -22,11 +22,11 @@ absl::optional<update_client::CrxComponent> GetComponent(
   return absl::nullopt;
 }
 
-std::vector<absl::optional<update_client::CrxComponent>> GetCrxComponents(
-    const base::flat_map<std::string, update_client::CrxComponent>&
+std::vector<absl::optional<ComponentRegistration>> GetCrxComponents(
+    const base::flat_map<std::string, ComponentRegistration>&
         registered_components,
     const std::vector<std::string>& ids) {
-  std::vector<absl::optional<update_client::CrxComponent>> components;
+  std::vector<absl::optional<ComponentRegistration>> components;
   for (const auto& id : ids)
     components.push_back(GetComponent(registered_components, id));
   return components;
