@@ -26,19 +26,19 @@ namespace device_event_log {
 
 namespace {
 
-const char* kLogLevelName[] = {"Error", "User", "Event", "Debug"};
+const char* const kLogLevelName[] = {"Error", "User", "Event", "Debug"};
 
-const char* kLogTypeNetworkDesc = "Network";
-const char* kLogTypePowerDesc = "Power";
-const char* kLogTypeLoginDesc = "Login";
-const char* kLogTypeBluetoothDesc = "Bluetooth";
-const char* kLogTypeUsbDesc = "USB";
-const char* kLogTypeHidDesc = "HID";
-const char* kLogTypeMemoryDesc = "Memory";
-const char* kLogTypePrinterDesc = "Printer";
-const char* kLogTypeFidoDesc = "FIDO";
-const char* kLogTypeSerialDesc = "Serial";
-const char* kLogTypeCameraDesc = "Camera";
+const char kLogTypeNetworkDesc[] = "Network";
+const char kLogTypePowerDesc[] = "Power";
+const char kLogTypeLoginDesc[] = "Login";
+const char kLogTypeBluetoothDesc[] = "Bluetooth";
+const char kLogTypeUsbDesc[] = "USB";
+const char kLogTypeHidDesc[] = "HID";
+const char kLogTypeMemoryDesc[] = "Memory";
+const char kLogTypePrinterDesc[] = "Printer";
+const char kLogTypeFidoDesc[] = "FIDO";
+const char kLogTypeSerialDesc[] = "Serial";
+const char kLogTypeCameraDesc[] = "Camera";
 
 enum class ShowTime {
   kNone,
@@ -77,7 +77,7 @@ std::string GetLogTypeString(LogType type) {
   return "Unknown";
 }
 
-LogType GetLogTypeFromString(const std::string& desc) {
+LogType GetLogTypeFromString(base::StringPiece desc) {
   std::string desc_lc = base::ToLowerASCII(desc);
   for (int i = 0; i < LOG_TYPE_UNKNOWN; ++i) {
     auto type = static_cast<LogType>(i);
@@ -116,7 +116,7 @@ std::string TimeWithMillieconds(const base::Time& time) {
                             exploded.millisecond);
 }
 
-#if defined(OS_POSIX)
+#if BUILDFLAG(IS_POSIX)
 std::string UnixTime(const base::Time& time) {
   base::Time::Exploded utc_exploded, exploded;
   time.UTCExplode(&utc_exploded);
@@ -151,7 +151,7 @@ std::string LogEntryToString(const DeviceEventLogImpl::LogEntry& log_entry,
   std::string line;
   if (show_time == ShowTime::kTimeWithMs)
     line += "[" + TimeWithMillieconds(log_entry.time) + "] ";
-#if defined(OS_POSIX)
+#if BUILDFLAG(IS_POSIX)
   if (show_time == ShowTime::kUnix)
     line += UnixTime(log_entry.time) + " ";
 #endif
@@ -160,7 +160,7 @@ std::string LogEntryToString(const DeviceEventLogImpl::LogEntry& log_entry,
   if (show_level) {
     const char* kLevelDesc[] = {"ERROR", "USER", "EVENT", "DEBUG"};
     line += std::string(kLevelDesc[log_entry.log_level]);
-#if defined(OS_POSIX)
+#if BUILDFLAG(IS_POSIX)
     if (show_time == ShowTime::kUnix) {
       // Format the level consistently with /var/log/messages.
       line += base::StringPrintf(" chrome[%d]", base::GetCurrentProcId());
@@ -248,24 +248,24 @@ void GetFormat(const std::string& format_string,
   *show_level = false;
   *format_json = false;
   while (tokens.GetNext()) {
-    std::string tok(tokens.token());
-    if (tok == "time")
+    base::StringPiece tok = tokens.token_piece();
+    if (tok == "time") {
       *show_time = ShowTime::kTimeWithMs;
-    if (tok == "unixtime") {
-#if defined(OS_POSIX)
+    } else if (tok == "unixtime") {
+#if BUILDFLAG(IS_POSIX)
       *show_time = ShowTime::kUnix;
 #else
       *show_time = ShowTime::kTimeWithMs;
 #endif
-    }
-    if (tok == "file")
+    } else if (tok == "file") {
       *show_file = true;
-    if (tok == "type")
+    } else if (tok == "type") {
       *show_type = true;
-    if (tok == "level")
+    } else if (tok == "level") {
       *show_level = true;
-    if (tok == "json")
+    } else if (tok == "json") {
       *format_json = true;
+    }
   }
 }
 
@@ -274,8 +274,8 @@ void GetLogTypes(const std::string& types,
                  std::set<LogType>* exclude_types) {
   base::StringTokenizer tokens(types, ",");
   while (tokens.GetNext()) {
-    std::string tok(tokens.token());
-    if (tok.substr(0, 4) == "non-") {
+    base::StringPiece tok = tokens.token_piece();
+    if (base::StartsWith(tok, "non-")) {
       LogType type = GetLogTypeFromString(tok.substr(4));
       if (type != LOG_TYPE_UNKNOWN)
         exclude_types->insert(type);
@@ -399,7 +399,7 @@ std::string DeviceEventLogImpl::GetAsString(StringOrder order,
   GetLogTypes(types, &include_types, &exclude_types);
 
   std::string result;
-  base::ListValue log_entries;
+  base::Value log_entries(base::Value::Type::LIST);
   if (order == OLDEST_FIRST) {
     size_t offset = 0;
     if (max_events > 0 && max_events < entries_.size()) {
@@ -428,7 +428,7 @@ std::string DeviceEventLogImpl::GetAsString(StringOrder order,
       if (entry.log_level > max_level)
         continue;
       if (format_json) {
-        log_entries.AppendString(LogEntryAsJSON(entry));
+        log_entries.Append(LogEntryAsJSON(entry));
       } else {
         result += LogEntryToString(entry, show_time, show_file, show_type,
                                    show_level);
@@ -444,7 +444,7 @@ std::string DeviceEventLogImpl::GetAsString(StringOrder order,
       if (entry.log_level > max_level)
         continue;
       if (format_json) {
-        log_entries.AppendString(LogEntryAsJSON(entry));
+        log_entries.Append(LogEntryAsJSON(entry));
       } else {
         result += LogEntryToString(entry, show_time, show_file, show_type,
                                    show_level);
